`timescale 1ns/1ns
module testbench;

reg i_clk;
reg i_reset;

reg i_tInput_valid_in;
wire o_tInput_stall_out;

reg i_tReset_valid_in;
wire o_tReset_stall_out;

wire o_tSum_valid_out;
reg i_tSum_stall_in;

reg [31:0] i_value;
wire [31:0] o_sum;

Accumulator acc0(
   .i_clk              ( i_clk              ),  
   .i_reset            ( i_reset            ),  

   .i_value            ( i_value            ),
   .o_sum              ( o_sum              ),

   .i_tInput_valid_in  ( i_tInput_valid_in  ),
   .o_tInput_stall_out ( o_tInput_stall_out ),

   .i_tReset_valid_in  ( i_tReset_valid_in  ),
   .o_tReset_stall_out ( o_tReset_stall_out ),

   .o_tSum_valid_out   ( o_tSum_valid_out   ),
   .i_tSum_stall_in    ( i_tSum_stall_in    )
);

always 
   #5 i_clk = ~i_clk;

`define TIMEOUT 100000
integer timeout_ctr = 0;
integer still_running = 1;

initial begin

   i_clk = 0;
   i_reset = 0;

   // Basic Reset
   i_tInput_valid_in = 0;
   i_tReset_valid_in = 0;
   i_tSum_stall_in = 0;
   i_value = 32'd0;

   // This resets the internal logic
   #100 i_reset = 1;

   // Now initialize the sum variable
   #10; i_tReset_valid_in = 1; #10; i_tReset_valid_in = 0;

   $display("-- starting accumulator --");
   
   while( still_running ) begin

      #10
      timeout_ctr = timeout_ctr + 1;
      if(timeout_ctr > `TIMEOUT) begin
         $display("!! timeout!");
         still_running = 0;
      end

      i_tInput_valid_in = 1;
      i_value = i_value + 1;
      #10
      i_tInput_valid_in = 0;

   end

   $finish;
   
end

always @(posedge o_tSum_valid_out)
   $display("sum = %d", o_sum);

endmodule
