`timescale 1ns/1ns
module testbench;

reg i_clk;
reg i_reset;

reg i_tInput_valid_in;
wire o_tInput_stall_out;

wire o_tOutput_valid_out;
reg i_tOutput_stall_in;

reg [15:0] i_x;
wire [15:0] o_result;

Quadratic quad0(
   .i_clk              ( i_clk              ),  
   .i_reset            ( i_reset            ),  

   .i_x                ( i_x                ),
   .o_result           ( o_result           ),

   .i_tInput_valid_in  ( i_tInput_valid_in  ),
   .o_tInput_stall_out ( o_tInput_stall_out ),

   .o_tOutput_valid_out( o_tOutput_valid_out),
   .i_tOutput_stall_in ( i_tOutput_stall_in )
);

always 
   #5 i_clk = ~i_clk;

`define TIMEOUT 1000
integer timeout_ctr = 0;
integer still_running = 1;
reg [15:0] tb_x_squared, tb_bx, tb_sum;

initial begin

   i_clk = 0;
   i_reset = 0;

   // Basic Reset
   i_tInput_valid_in = 0;
   i_tOutput_stall_in = 0;
   i_x = 16'd0;

   // This resets the internal logic
   #100 i_reset = 1;

   $display("-- starting accumulator --");
   
   while( still_running ) begin

      #10
      timeout_ctr = timeout_ctr + 1;
      if(timeout_ctr > `TIMEOUT) begin
         $display("!! timeout!");
         still_running = 0;
      end

      i_tInput_valid_in = 1;
      i_x = i_x + 1;
      tb_x_squared = i_x * i_x;
      tb_bx = i_x * 20;
      tb_sum = tb_bx + tb_x_squared;
      #10
      i_tInput_valid_in = 0;

      $display("              result should be %d", tb_sum);

   end

   $stop;
   
end

/* Note: This output will appear delayed. This is because it takes
   a few cycles to propagate through the design. */
always @(posedge o_tOutput_valid_out) begin
   $display("result = %d", o_result);
end

endmodule
