`timescale 1ns/1ns
module testbench;

reg i_clk;
reg i_reset = 0;

reg i_tReset_valid_in = 0;
wire o_tReset_stall_out;

wire o_tReadMemory_valid_out;
wire i_tReadMemory_stall_in;

wire o_tMax_valid_out;
reg i_tMax_stall_in = 0;

wire i_tMemoryValid_valid_in;
wire o_tMemoryValid_stall_out;

reg [31:0] i_max = 0;
wire [31:0] o_max_out = 0;
reg [7:0] i_reset_addr = 0;
wire [31:0] i_mem_data_in;

wire [7:0] o_mem_addr_out;
wire [7:0] address_loop;

RomMax rommax(
   .i_clk(i_clk),
   .i_reset(i_reset),

   .i_reset_addr(i_reset_addr),
   .i_max(i_max),

   .o_max_out(o_max_out),

   .o_addr( address_loop ),
   .i_addr( address_loop ),

   .i_tReadMemory_stall_in(i_tReadMemory_stall_in),
   .o_tReadMemory_valid_out(o_tReadMemory_valid_out),

   .i_mem_data_in(i_mem_data_in),
   //o_mem_addr_out(o_mem_addr_out),

   .i_tMemoryValid_valid_in(i_tMemoryValid_valid_in),
   .o_tMemoryValid_stall_out(o_tMemoryValid_stall_out),

   .o_tReset_stall_out(o_tReset_stall_out),
   .i_tReset_valid_in(i_tReset_valid_in),

   .i_tMax_stall_in(i_tMax_stall_in),
   .o_tMax_valid_out(o_tMax_valid_out)
);

assign o_mem_addr_out = address_loop;

elasticROM #(
   .ADDRESS_WIDTH(8),
   .DATA_WIDTH(32),
   .DEPTH(256),
   .FILE("testrom.hex")
) rom (
   .valid_in( o_tReadMemory_valid_out ),
   .stall_out( i_tReadMemory_stall_in ),
   .valid_out( i_tMemoryValid_valid_in ),
   .stall_in( o_tMemoryValid_stall_out ),
   .addr( o_mem_addr_out ),
   .data_out( i_mem_data_in )
);

always 
   #5 i_clk = ~i_clk;

`define TIMEOUT 1000
integer timeout_ctr = 0;
integer still_running = 1;

initial begin

   i_clk = 0;
   i_reset = 0;

   // Basic Reset

   // This resets the internal logic
   #100 i_reset = 1;

   // Now initialize the sum variable
   #10; i_tReset_valid_in = 1; #10; i_tReset_valid_in = 0;

   $display("-- starting accumulator --");
   
   while( still_running ) begin

      #10
      timeout_ctr = timeout_ctr + 1;
      if(timeout_ctr > `TIMEOUT) begin
         $display("!! timeout!");
         still_running = 0;
      end

   end

   $stop;
   //$finish;
   
end

always @(posedge o_tMax_valid_out) begin
   $display("max detected: %d / %h", o_max_out, o_max_out);
end

endmodule
