/******************************************************************************
;   Code for application report slaa221 - "CRC Implementation with MSP430"
;
;   E.Lenchak
;   Texas Instruments, Inc
;   March 2004
;   Built with IAR Embedded Workbench Version: 3.20A
;******************************************************************************
; THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
; REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY,
; INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
; FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR
; COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE.
; TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET
; POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY
; INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR
; YOUR USE OF THE PROGRAM.
;
; IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
; CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY
; THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED
; OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT
; OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM.
; EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF
; REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS
; OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF
; USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S
; AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF
; YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS
; (U.S.$500).
;
; Unless otherwise stated, the Program written and copyrighted
; by Texas Instruments is distributed as "freeware".  You may,
; only under TI's copyright in the Program, use and modify the
; Program without any charge or restriction.  You may
; distribute to third parties, provided that you transfer a
; copy of this license to the third party and the third party
; agrees to these terms by its first use of the Program. You
; must reproduce the copyright notice and any other legend of
; ownership on each copy or partial copy, of the Program.
;
; You acknowledge and agree that the Program contains
; copyrighted material, trade secrets and other TI proprietary
; information and is protected by copyright laws,
; international copyright treaties, and trade secret laws, as
; well as other intellectual property laws.  To protect TI's
; rights in the Program, you agree not to decompile, reverse
; engineer, disassemble or otherwise translate any object code
; versions of the Program to a human-readable form.  You agree
; that in no event will you alter, remove or destroy any
; copyright notice included in the Program.  TI reserves all
; rights not specifically granted under this license. Except
; as specifically provided herein, nothing in this agreement
; shall be construed as conferring by implication, estoppel,
; or otherwise, upon you, any license or other right under any
; TI patents, copyrights or trade secrets.
;
; You may not use the Program in non-TI devices.
;
;******************************************************************************/

#include "stdio.h"
#include "crc.h"

/*************************
	PROTOTYPES
*************************/

unsigned short crc16MakeBitwise(unsigned short, unsigned short, unsigned char*, unsigned int);
unsigned short crc16MakeBitwise2(unsigned short, unsigned short, unsigned char*, unsigned int);
  
unsigned long crc32MakeBitwise(unsigned long, unsigned long, unsigned char*, unsigned int);
unsigned long crc32MakeBitwise2(unsigned long, unsigned long, unsigned char*, unsigned int);
  
unsigned short crc16MakeTableMethod(unsigned short, TBL_MEM unsigned short*, unsigned char *, unsigned int);
  
unsigned long crc32MakeTableMethod(unsigned long, TBL_MEM unsigned long*, unsigned char*, unsigned int);
unsigned long crc32rMakeTableMethod(unsigned long, TBL_MEM unsigned long*, unsigned char*, unsigned int);

void crc16BuildTable(unsigned short*, unsigned short);
void crc32BuildTable(unsigned long*, unsigned long);

unsigned long bitReflect(unsigned long, unsigned int);

/*************************
	EXTERNALS
*************************/

//extern TBL_MEM unsigned short crc16Table[CRC_TABLE_SIZE];
//extern TBL_MEM unsigned long crc32Table[CRC_TABLE_SIZE];
//extern TBL_MEM unsigned short crc16rTable[CRC_TABLE_SIZE];
//extern TBL_MEM unsigned long crc32rTable[CRC_TABLE_SIZE];

#ifdef __cplusplus
}
#endif

/*************************
	GLOBALS
*************************/

unsigned char msg[CRC_MSG_MAXSIZE] = CRC_TEST_SEQ "5555"; //added 5's (non-0's) to check for overflow

/*************************
	MAIN
*************************/

int 
main(int argc, char* argv[])
{
	char buf[256];

	unsigned int length;
	unsigned long crc32;
	unsigned short crc16;

	puts("Starting CRC Test");
	
	strcpy(buf, "Message: ");
	strcpy(&buf[9], CRC_TEST_SEQ);
	strcpy(&buf[18], " (len = ");
	length = sizeof(CRC_TEST_SEQ)-1;
	itoa(length, &buf[26]);
	buf[27] = ')';
	buf[28] = 0;
	
	puts(buf);

	// calculate CRC brute force alg (optimized implementation for C)
	crc16 = crc16MakeBitwise(CRC16_INIT_REM, CRC16_POLY, msg, length);

	strcpy(buf, "CRC16_bit: 0x");
	itoa(crc16, &buf[13], 16);
	puts(buf);

	// calculate CRC brute force alg version #2 (same implementation as assembly routine)
/*	crc16 = crc16MakeBitwise2(CRC16_INIT_REM, CRC16_POLY, msg, length);
	printf("CRC16_bit: 0x%X(N)\n", crc16);

	// calculate CRC with table method
	crc16 = crc16MakeTableMethod(CRC16_INIT_REM, crc16Table, msg, length);
	printf("CRC16_tab: 0x%X(N)\n", crc16);

	// calculate reflected CRC with table method
	crc16 = crc16rMakeTableMethod(CRC16R_INIT_REM, crc16rTable, msg, length);
	printf("CRC16_tab: 0x%X(R)\n", crc16);
*/
	// print expected CRCs
	strcpy(buf, "CRC16_exp: 0x");
	itoa(CRC16_CHECK, &buf[13], 16);
	puts(buf);
/*	printf("CRC16_exp: 0x%X(R)\n\n", CRC16R_CHECK);	

	// calculate CRC brute force alg (optimized implementation for C)
	crc32 = crc32MakeBitwise(CRC32_INIT_REM, CRC32_POLY, msg, length);
	printf("CRC32_bit: 0x%lX(N)\n", crc32);

	// calculate CRC brute force alg version #2 (same implementation as assembly routine)
	crc32 = crc32MakeBitwise2(CRC32_INIT_REM, CRC32_POLY, msg, length);
	printf("CRC32_bit: 0x%lX(N)\n", crc32);

	// calculate CRC with table method
	crc32 = crc32MakeTableMethod(CRC32_INIT_REM, crc32Table, msg, length);
	printf("CRC32_tab: 0x%lX(N)\n", crc32);

	// calculate reflected CRC with table method
	crc32 = crc32rMakeTableMethod(CRC32R_INIT_REM, crc32rTable, msg, length);
	printf("CRC32_tab: 0x%lX(R)\n", crc32);

	// print expected CRCs
	printf("CRC32_exp: 0x%lX(N)\n", CRC32_CHECK);	
	printf("CRC32_exp: 0x%lX(R)\n\n", CRC32R_CHECK);	
*/

	puts("Ending CRC Test");

	return 0;
}

/************************* END MAIN ************************************/
