
import sys

from optparse import OptionParser


start_address = 0x4000
end_address = 0xFFEF
granularity = 2

SINGLE_MAP = { 0: "RRC",
			   1: "SWPB",
			   2: "RRA",
			   3: "SXT",
			   4: "PUSH",
			   5: "CALL",
			   6: "RETI"}
			   
DOUBLE_MAP = { 4: "MOV",
			   5: "ADD",
			   6: "ADDC",
			   7: "SUBC",
			   8: "SUB",
			   9: "CMP",
			   10: "DADD",
			   11: "BIT",
			   12: "BIC",
			   13: "BIS",
			   14: "XOR",
			   15: "AND"}
			   
JUMP_MAP = { 0: "JNE",
			 1: "JEQ",
			 2: "JNC",
			 3: "JC",
			 4: "JN",
			 5: "JGE",
			 6: "JL",
			 7: "JMP"}
			 

def reg(num, mode):
	if num == 0:
		reg = "PC"
	elif num == 1:
		reg = "SP"
	else:
		reg = "R" + str(num)
		
	if num == 2:
		if mode == 0:
			return "SR"
		elif mode == 1:
			return "&"
		elif mode == 2:
			return "#4"
		elif mode == 3:
			return "#8"
		
	elif num == 3:
		if mode == 0:
			return "#0"
		elif mode == 1:
			return "#1"
		elif mode == 2:
			return "#2"
		elif mode == 3:
			return "#-1"
	else:
		if mode == 1:
			reg = "(" + reg + ")"
		elif mode == 2:
			reg = "@" + reg
		elif mode == 3:
			reg = "@" + reg + "+"
		return reg


def ilen(ascii):
	instr = int(ascii, 16)
	ilen = 1
	# jump
	if (instr & 0xE000) >> 13 == 1:
		pass
	
	# single
	elif (instr & 0xF000) >> 12 == 1:
		ad = (instr >> 4) & 0x3
		reg = instr & 0xF
		if (ad == 1 and reg != 3) or (reg == 0 and ad == 3):
			ilen += 1
	
	# double
	else:
		ad = (instr >> 7) & 0x1
		if ad:
			ilen += 1
		as_ = (instr >> 4) & 0x3
		sreg = (instr >> 8) & 0xF
		if as_ == 1 and sreg != 2 and sreg != 3:
			ilen += 1
		elif as_ == 3 and sreg == 0:
			ilen += 1

	return ilen		

		
def comment(ascii):
	instr = int(ascii, 16)
	if instr == 0:
		return "NOP"
		
	cstr = ""
	if (instr & 0xE000) >> 13 == 1:
		condition = (instr >> 10) & 0x7		
		offset = (instr & 0x3FF) << 1
		#print offset
		# neg
		if offset & 0x400:	
			#print "neg"
			offset = (~offset & 0x7FF) + 1
			offset = -offset
		cstr = JUMP_MAP[condition]
		cstr += " " + str(offset)
				
	elif (instr & 0xF000) >> 12 == 1:
		opcode = (instr >> 7) & 0x7
		bw = (instr >> 6) & 0x1
		ad = (instr >> 4) & 0x3
		dreg = instr & 0xF
		if opcode > 6:
			print ascii
		cstr = SINGLE_MAP[opcode]
		if bw:
			cstr += ".B"
		cstr += " "
		cstr += reg(dreg, ad)
		
	else:
		opcode = instr >> 12
		sreg = (instr >> 8) & 0xF
		ad = (instr >> 7) & 0x1
		bw = (instr >> 6) & 0x1
		as_ = (instr >> 4) & 0x3
		dreg = instr & 0xF
		
		# Must be data
		if not opcode in DOUBLE_MAP:	
			return ""
			
		#print ascii
		#print opcode, sreg, as_, bw, dreg, ad
		cstr = DOUBLE_MAP[opcode]
		if bw:
			cstr += ".B"
		 
		cstr += " " + reg(sreg, as_)
		cstr += ", " + reg(dreg, ad)
		
	return cstr
		

	
if __name__ == "__main__":
	usage = "usage: python ihex_to_ver.py [-s start -e end] filename"
	usage = "usage: %prog [options] filename"
	parser = OptionParser(usage=usage)
	parser.add_option("-s", "--start", action="store", dest="start", default="0000", help="start address")
	parser.add_option("-e", "--end", action="store", dest="end", default="FFFF", help="end address")
	parser.add_option("-c", "--comment", action="store_true", dest="comment", default=False, help="comment assembly")
	parser.add_option("-a", "--address", action="store_true", dest="address", default=False, help="show address")
	parser.add_option("-w", "--width", action="store", dest="width", default=2, help="byte width")
	parser.add_option("-b", "--banks", action="store", dest="banks", default=1, help="number of banks")
	parser.add_option("-n", "--number", action="store", dest="bank", default=0, help="bank number")
	(options, args) = parser.parse_args()
	
	if len(args) == 0:
		parser.print_help()
		sys.exit(0)

	print "// Converting %s..." % args[0]
	f = open(args[0], 'r')

	options.start = int(options.start, 16)
	options.end = int(options.end, 16)
	options.banks = int(options.banks)
	options.bank = int(options.bank)
	granularity = int(options.width)
	
	address = 0
	skip = 0
	code = ""
	
	for line in f:
		if line[0] != ':':
			print "Error: Invalid ihex file format (line %i)." % i
			break
		line = line.strip()
		
		count = int(line[1:3], 16) / 2
		addr = int(line[3:7], 16)
		type = line[7:9]
		
		if type == "00":
			if (address + (len(code) / 2)) != addr:
				code = "00" * (addr - address - (len(code)/2)) + code
			
			# pad any zeros as necessary
			#while address < addr and address <= options.end:					
			#	if options.banks == 1 or int(address / granularity) % options.banks == options.bank:
			#		if address % granularity == 0:
			#			print "   -- %x %x -- %i %s --" % (address, addr, len(code), code)
			#			if options.address:
			#				sys.stdout.write("%x: " % address)
			#		
			#		sys.stdout.write("00")
			#	address += 1

			# construct code line in correct order
			word = 0
			while word < count:
				data = line[9+(4*word):9+(4*word)+4]
				data = data[2:] + data[:2]
				word += 1
				code = data + code
			#print "\r\"" + code + "\"", count
			
			while len(code) >= (2 * granularity):
				if address <= options.end and address >= options.start:
					if options.banks == 1 or int(address / granularity) % options.banks == options.bank:
						if address % granularity == 0:
							print ""
							if options.address:
								sys.stdout.write("%x: " % address)
				
						sys.stdout.write(code[-(2 * granularity):])
					
					if options.banks == 1 or int(address / granularity) % options.banks == options.bank:
						if options.comment and granularity == 2 and skip == 0:
							instr = code[-(2 * granularity):]
							sys.stdout.write("\t// ")
							sys.stdout.write(comment(instr))
							skip = ilen(instr)
						skip -= 1
			
				address += granularity
				code = code[:-(2 * granularity)]
			
	print ""

	f.close()
	
	
	