
from Instruction import Instruction
from Status import Status


from Int import Int, GlobalIn, GlobalOut
					

class ExecUnit:
	def __init__(self):
		# Instruction Interface
		self.pc = GlobalIn(16)	
		self.sp = GlobalIn(16)
		self.sr = GlobalIn(16)
		self.thread = GlobalIn(2)
		
		#Interrupt
		self.vector = GlobalIn(4)
		
		# Register File Interface
		self.rReadThreadOut = GlobalOut(2)
		self.rReadAddrOut = GlobalOut(4)
		self.rReadDataIn = GlobalIn(16)
		self.rReadOpTagOut = GlobalOut(1)
		self.rReadOpTagIn = GlobalIn(1)

		self.rWriteThreadOut = GlobalOut(2)
		self.rWriteAddrOut = GlobalOut(4)
		self.rWriteDataOut = GlobalOut(16)
		self.rWriteOpTagOut = GlobalOut(2)
		self.rWriteOpTagIn = GlobalIn(2)
		
		# Memory Interface
		self.mReadAddrOut = GlobalOut(16)
		self.mReadDataIn = GlobalIn(16)
		self.mReadByte = GlobalOut(1)
		self.mReadOpTagOut = GlobalOut(3)
		self.mReadOpTagIn = GlobalIn(3)
		
		self.mWriteAddrOut = GlobalOut(16)
		self.mWriteDataOut = GlobalOut(16)
		self.mWriteByte = GlobalOut(1)
		self.mWriteOpTagOut = GlobalOut(3)
		self.mWriteOpTagIn = GlobalIn(3)

		# Program Memory Interface
		self.romReadAddrOut = GlobalOut(16)
		self.romReadDataIn = GlobalIn(48)
		

		# Internal Data
		self.status = Status()
		self.ir = Instruction()

		self.value = Int(16)
		
		self.ismems = Int(1)
		self.memaddrs = Int(16)
		self.memaddrd = Int(16)
		
		self.ops = Int(16)
		self.opd = Int(16)
		self.result = Int(16)
		
		self.pco = GlobalOut(16)
		self.sro = GlobalOut(16)
		self.spo = GlobalOut(16)
		self.threado = GlobalOut(2)
		

	# ------------------------------------------------------------------------------
	# INTERRUPT
	# ------------------------------------------------------------------------------

	def interrupt(self, trigger="tInterrupt"):
		self.status = self.sr
		
		# use regular memory interface
		self.mReadAddrOut = (self.vector << 1) + 0xE0
		self.mReadAddrOut = self.mReadAddrOut | 0xFF00
		self.mReadByte = 0
		self.mReadOpTagOut = 7
		
		create(tMemRead)
		create(tIntMemReadWait)


	def intVectorRead(self, trigger="tIntMemReadDone and tIntMemReadWait"):
		self.pc = self.value

		if self.vector != 0xF:
			self.sp = self.sp - 2
			self.mWriteAddrOut = self.sp
			self.mWriteDataOut = self.pc
			self.mWriteByte = 0
			self.mWriteOpTagOut = 2
	
			create(tMemWrite)
			create(tIntPCMemWriteWait)

		else:
			self.status = 0
			create(tCommit)
			

	def pcIntPushDone(self, trigger="tIntPCMemWriteDone	and tIntPCMemWriteWait"):
		self.sp = self.sp - 2
		self.mWriteAddrOut = self.sp
		self.mWriteDataOut = self.status
		self.mWriteByte = 0
		self.mWriteOpTagOut = 3
		create(tMemWrite)
		create(tIntSRMemWriteWait)
		
		
	def intDone(self, trigger="tIntSRMemWriteDone and tIntSRMemWriteWait"):
		create(tCommit)


	# ------------------------------------------------------------------------------
	# SOURCE RETRIEVAL
	# ------------------------------------------------------------------------------
	def cycle(self, trigger="tCycle"):
		self.romReadAddrOut = self.pc
		create(tRomRead)
		create(tFetch)

		self.status = self.sr
		
		
	def romReadDone(self, trigger="tRomReadDone"):
		self.ir = self.romReadDataIn
		create(tInstructionReady)
		

	def decode(self, trigger="tFetch and tInstructionReady"):
		offset = Int(16)
		offset = 0

		# Perform Jump ops
		if self.ir.is_jump():
			jmp = Int(1)
			jmp = 0
			if self.ir.jump_c() == 0:
				if self.status.Z() == 0:
					jmp = 1
			if self.ir.jump_c() == 1:
				if self.status.Z() == 1:
					jmp = 1
			if self.ir.jump_c() == 2:
				if self.status.C() == 0:
					jmp = 1
			if self.ir.jump_c() == 3:
				if self.status.C() == 1:
					jmp = 1
			if self.ir.jump_c() == 4:
				if self.status.N() == 1:
					jmp = 1
			if self.ir.jump_c() == 5:
				if self.status.N() == self.status.V():
					jmp = 1
			if self.ir.jump_c() == 6:
				if self.status.N() != self.status.V():
					jmp = 1
			if self.ir.jump_c() == 7:
				jmp = 1

			if jmp:			
				offset = self.ir.jump_offset()	# rtl_writer deficiency so write to variable first
			
			create(tCommit)
			
		# Handle Single/Double operand opcodes
		else:
			# Check for source operand mode and register
			rs = Int(4)	# reg
			ms = Int(2) # mode
			if self.ir.is_single():
				rs = self.ir.single_dsreg()
				ms = self.ir.single_ad()
				
			elif self.ir.is_double():
				rs = self.ir.double_sreg()
				ms = self.ir.double_as()
			
			# PC
			if rs == 0:
				self.ops = self.pc
				create(tRsIsInternal)
			
			# SP
			elif rs == 1:
				self.ops = self.sp
				create(tRsIsInternal)
				
			# Constant Generator
			elif rs == 2 or rs == 3:
				if rs == 2:
					# status register
					if ms == 0:
						self.ops = self.status
					# absolute
					elif ms == 1:
						self.ops = 0
					elif ms == 2:
						self.ops = 4
					elif ms == 3:
						self.ops = 8
					
					create(tRsIsInternal)
					
				elif rs == 3:
					if ms == 0:
						self.ops = 0
					if ms == 1:
						self.ops = 1
					if ms == 2:
						self.ops = 2
					if ms == 3:
						self.ops = 0xffff
						
					create(tRsIsInternal)
			
			# Handle as standard register access
			else:	 
				self.rReadAddrOut = rs
				self.rReadThreadOut = self.thread
				self.rReadOpTagOut = 0
				create(tRegRead)
				create(tRsRegWait)	# flag to wait for read to complete -- also route internal data


		self.pc = self.pc + offset + 2
	
	
	# ------------------------------------------------------------------------------
	# SOURCE REGISTER RETRIEVAL
	# ------------------------------------------------------------------------------
	def rsRegDone(self, trigger="tRsRegReadDone and tRsRegWait"):
		self.ops = self.value
		
		create(tRsRegIsDone)
		
	def rsGetRegDone(self, trigger="tRsIsInternal or tRsRegIsDone"):
		rs = Int(4) # reg
		ms = Int(2) # mode
		bw = Int(1) # byte
		if self.ir.is_single():
			rs = self.ir.single_dsreg()
			ms = self.ir.single_ad()
			bw = self.ir.single_bw()
			
		elif self.ir.is_double():
			rs = self.ir.double_sreg()
			ms = self.ir.double_as()
			bw = self.ir.double_bw()
			
		self.ismems = 0

		# Direct or CG values have are ready
		if ms == 0 or rs == 3 or (rs == 2 and ms != 1):
			create(tRsRegDone)
		
		# Indirect values need to be fetched from memory
		else:
			self.memaddrs = self.ops
			self.ismems = 1

			#optimize!
			if ms == 1 or (ms == 3 and rs == 0):
				self.pc = self.pc + 2

			if ms == 1:
				# If indirect mode (&) then handle specially
				if rs == 2:
					self.memaddrs = self.ir.extension1()
					
				else:
					self.memaddrs = self.memaddrs + self.ir.extension1()	# add extension
			
				# increment along the way in case it becomes an operand -- can optimize???
				#self.pc = self.pc + 2

			# Optimize load immediate PC+
			if ms == 3 and rs == 0:
				self.ops = self.ir.extension1()
				#self.pc = self.pc + 2
				
				create(tRsRegDone)
				
			else:
				self.mReadAddrOut = self.memaddrs
				self.mReadOpTagOut = 2
				self.mReadByte = bw
	
				create(tRsMemWait)
				create(tMemRead)

			
				# Post Increment
				if ms == 3:
					#if rs == 0:
					#	self.pc = self.pc + 2
					#	create(tNoInc)
						
					if rs == 1:
						self.sp = self.sp + 2
						create(tNoInc)
		
					# Write back to register file
					elif rs[2] or rs[3]:
					#elif rs > 3:
						self.rWriteOpTagOut = 2
						self.rWriteAddrOut = rs
						self.rWriteThreadOut = self.thread
						add = Int(2)
						add = 2
						if bw and rs != 0 and rs != 1:
							add = 1
						# optimize
						#	self.rWriteDataOut = self.ops + 1
						#else:
						#	self.rWriteDataOut = self.ops + 2
						self.rWriteDataOut = self.ops + add
						
						create(tRegWrite)
				else:
					create(tNoInc)
		
	# ------------------------------------------------------------------------------
	# END SOURCE REGISTER RETRIEVAL
	# ------------------------------------------------------------------------------

	# ------------------------------------------------------------------------------
	# SOURCE MEMORY RETRIEVAL
	# ------------------------------------------------------------------------------

	def rsMemReadDone(self, trigger="tRsMemReadDone and tRsMemWait and (tIncRegWriteDone or tNoInc)"):
		self.ops = self.value
		
		create(tRsRegDone)
		
	# ------------------------------------------------------------------------------
	# END SOURCE MEMORY RETRIEVAL
	# ------------------------------------------------------------------------------
	
	# ------------------------------------------------------------------------------
	# END SOURCE RETRIEVAL
	# ------------------------------------------------------------------------------
	
	# ------------------------------------------------------------------------------
	# PERFORM SINGLE OP
	# ------------------------------------------------------------------------------
	# ------------------------------------------------------------------------------
	# DEST RETRIEVAL
	# ------------------------------------------------------------------------------
	def rsLatchSep(self, trigger="tRsRegDone"):
		create(tRsRegLatch)
		
	def rsReady(self, trigger="tRsRegLatch"):
		# Perform single operand instructions
		if self.ir.is_single():
			# Should we write back after this op?
			wr = Int(1)
			wr = 1
			
			# RRC(.B)
			if self.ir.single_opcode() == 0:
				c = Int(1)
				c = self.ops[0]
				self.result = self.ops >> 1
				if self.ir.single_bw() == 1:
					self.result = self.result & 0x7F
					self.result = self.result | (self.status.C() << 7)
				else:
					self.result = self.result | (self.status.C() << 15)

				self.status[0] = c
				self.status[1] = (self.result == 0)
				self.status[2] = self.result[15]
				self.status[8] = 0
				
			# SWPB
			if self.ir.single_opcode() == 1:
				self.result = (self.ops[0:8] << 8) | (self.ops[8:16] >> 8)
			
			# RRA(.B)
			if self.ir.single_opcode() == 2:
				self.status[0] = self.ops[0]

				if self.ir.single_bw() == 1:
				#wrong
				#	self.result = (self.ops & 0x80) | (self.ops[1:8] >> 1)
				#added by wei
					self.result = (self.ops & 0x80) | (self.ops[0:8] >> 1)
					self.status[2] = self.result[7]
				else:
				#wrong
				#	self.result = (self.ops & 0x8000) | (self.ops[1:16] >> 1)
				#added by wei
					self.result = (self.ops & 0x8000) | (self.ops[0:16] >> 1)
					self.status[2] = self.result[15]

				self.status[1] = (self.result == 0)
				self.status[8] = 0
				
			# SXT
			if self.ir.single_opcode() == 3:
				self.result = (self.ops[7] << 8) or \
						   	  (self.ops[7] << 7) or \
						   	  (self.ops[7] << 6) or \
						   	  (self.ops[7] << 5) or \
							  (self.ops[7] << 4) or \
							  (self.ops[7] << 3) or \
							  (self.ops[7] << 2) or \
							  (self.ops[7] << 1) or \
							   self.ops[0:8]

				self.status[0] = (self.result != 0)
				self.status[1] = (self.result == 0)
				self.status[2] = self.result[15]
				self.status[8] = 0

			
			# PUSH(.B)
			if self.ir.single_opcode() == 4:
				self.sp = self.sp - 2
				self.mWriteAddrOut = self.sp
				self.mWriteDataOut = self.ops
				self.mWriteByte = self.ir.single_bw()
				self.mWriteOpTagOut = 0
				create(tMemWrite)
				create(tCommitSingleMemWait)
				wr = 0
			
			# CALL
			if self.ir.single_opcode() == 5:
				self.sp = self.sp - 2
				self.mWriteAddrOut = self.sp
				self.mWriteDataOut = self.pc
				self.mWriteByte = 0
				self.mWriteOpTagOut = 0
				create(tMemWrite)
				create(tCommitSingleMemWait)	# check for instruction type when this returns and update pc accordingly
				wr = 0	
				
			# RETI
			if self.ir.single_opcode() == 6:
				self.mReadAddrOut = self.sp
				self.mReadByte = 0
				self.mReadOpTagOut = 5
				create(tMemRead)
				create(tRetiSrMemWait)

				self.sp = self.sp + 2
				wr = 0
		
		
			# Need to writeback the result if we've got it
			if wr == 1:
				if self.ismems == 1:
					self.mWriteAddrOut = self.memaddrs
					self.mWriteDataOut = self.result
					self.mWriteByte = self.ir.single_bw()
					self.mWriteOpTagOut = 0
					create(tMemWrite)
					create(tCommitSingleMemWait)
					
				else:
					self.rWriteAddrOut = self.ir.single_dsreg()
					self.rWriteThreadOut = self.thread
					self.rWriteDataOut = self.result
					self.rWriteOpTagOut = 0
					create(tRegWrite)
					create(tCommitSingleRegWait)
		
		# Fetch destination operand for instruction
		else:
			# MOV
			# - special case of single operand instruction
			# - only with direct register access (no need to fetch operand)
			if self.ir.double_opcode() == 4 and self.ir.double_ad() == 0:
				if self.ir.double_dreg() == 0:
					self.pc = self.ops
					create(tCommit)
					
				elif self.ir.double_dreg() == 1:
					self.sp = self.ops
					create(tCommit)
					
				elif self.ir.double_dreg() == 2:
					self.status = self.ops
					create(tCommit)
					
				else:
					self.rWriteAddrOut = self.ir.double_dreg()
					self.rWriteThreadOut = self.thread
					self.rWriteDataOut = self.ops
					self.rWriteOpTagOut = 1
					create(tRegWrite)
					create(tCommitDoubleRegWait)			
	
			# Get register
			else:
				rd = Int(5)
				md = Int(1)
				rd = self.ir.double_dreg()
				md = self.ir.double_ad()

				# PC
				if rd == 0:
					self.opd = self.pc
					create(tRdIsInternal)
				
				# SP
				elif rd == 1:
					self.opd = self.sp
					create(tRdIsInternal)
				
				# Constant Generator
				elif rd == 2:
					if md == 0:
						self.opd = self.status
					else:
						self.opd = 0

					create(tRdIsInternal)
						
				elif rd == 3:
					self.opd = 0
					create(tRdIsInternal)
					
				# General register get
				else:
					self.rReadAddrOut = self.ir.double_dreg()
					self.rReadThreadOut = self.thread
					self.rReadOpTagOut = 1
					create(tRegRead)
					create(tRdRegWait)
	
	
	def retiSrPopDone(self, trigger="tRetiSrMemReadDone and tRetiSrMemWait"):
		self.status = self.value
		self.mReadAddrOut = self.sp
		self.mReadByte = 0
		self.mReadOpTagOut = 6
		create(tMemRead)
		create(tRetiPcMemWait)

		self.sp = self.sp + 2

	def retiPcPopDone(self, trigger="tRetiPcMemReadDone and tRetiPcMemWait"):
		self.pc = self.value

		create(tCommit)
		

	# ------------------------------------------------------------------------------
	# END PERFORM SINGLE OP
	# ------------------------------------------------------------------------------

	# ------------------------------------------------------------------------------
	# DEST REGISTER RETRIEVAL
	# ------------------------------------------------------------------------------
	
	def rdRegDone(self, trigger="tRdRegReadDone and tRdRegWait"):
		self.opd = self.value
		
		create(tRdRegIsDone)
		

	def rdGetRegDone(self, trigger="tRdIsInternal or tRdRegIsDone"):				
		# Direct access
		if self.ir.double_ad() == 0:
			create(tRdRegDone)
		
		# Indexed access
		else:
			self.memaddrd = self.opd

			ms = Int(2) # mode
			ms = self.ir.double_as()
			rs = Int(4) # reg
			rs = self.ir.double_sreg()
			# Optimize for immediate load PC+

			# optimize
			ext = Int(16)
			# commentted by wei
			#if (ms == 1 and rs == 2) or (ms == 3 and rs == 0):
			#added by wei
			if (ms == 1 and rs == 0) or (ms == 1 and rs == 1) or (ms == 1 and rs > 3) or (ms == 1 and rs == 2) or (ms == 3 and rs == 0):
				ext = self.ir.extension2()				
			else:
				ext = self.ir.extension1()

			self.memaddrd = self.memaddrd + ext

			self.pc = self.pc + 2

			# MOV
			# - special case of single operand instruction
			if self.ir.double_opcode() == 4:
				create(tRdRegDone)
			
			else:
				self.mReadAddrOut = self.memaddrd
				self.mReadByte = self.ir.double_bw()
				self.mReadOpTagOut = 4
				create(tMemRead)
				create(tRdMemWait)

	# ------------------------------------------------------------------------------
	# END DEST REGISTER RETRIEVAL
	# ------------------------------------------------------------------------------

	# ------------------------------------------------------------------------------
	# SOURCE MEMORY RETRIEVAL
	# ------------------------------------------------------------------------------

	def rdMemReadDone(self, trigger="tRdMemReadDone and tRdMemWait"):
		self.opd = self.value
		
		create(tRdRegDone)
		
	# ------------------------------------------------------------------------------
	# END DEST MEMORY RETRIEVAL
	# ------------------------------------------------------------------------------

	# ------------------------------------------------------------------------------
	# END DEST RETRIEVAL
	# ------------------------------------------------------------------------------

	# ------------------------------------------------------------------------------
	# PERFORM DOUBLE OP
	# ------------------------------------------------------------------------------

	def rdReady(self, trigger="tRdRegDone"):
		wr = Int(1)
		wr = 1

		# optimze!!!
		opa = Int(16)
		opb = Int(16)
		add = Int(1)
		add = 0
		sub = Int(1)
		sub = 0
		c = Int(1)
		c = 0
		
		# MOV
		if self.ir.double_opcode() == 4:
			self.result = self.ops
			
		opa = self.ops
		opb = self.opd

		# ADD
		if self.ir.double_opcode() == 5:
			add = 1

		# ADDC
		if self.ir.double_opcode() == 6:
			add = 1
			c = self.status.C()

		# SUBC
		if self.ir.double_opcode() == 7:
			sub = 1
			c = self.status.C()

		# SUB / CMP
		if self.ir.double_opcode() == 8 or self.ir.double_opcode() == 9:
			sub = 1
			c = 1


		# Extract ADD operand
		if add or sub:
			if sub:
				opa = ~opa
			self.result = opa + opb + c


		# ADD / ADDC / SUB / SUBC / CMP
		if self.ir.double_opcode() == 5 or \
		   self.ir.double_opcode() == 6 or \
		   self.ir.double_opcode() == 7 or \
		   self.ir.double_opcode() == 8 or \
		   self.ir.double_opcode() == 9:
			
			if self.ir.double_bw() == 1:
				self.status[0] = self.result[8]
				self.status[2] = self.result[7]
				self.status[8] = ((opa[7] & opb[7]) & ~self.result[7]) | ((~opa[7] & ~opb[7]) & self.result[7])

				self.result = self.result & 0xFF

			else:					
				# commented by wei
				self.status[0] = ((opa[15] | opb[15]) & ~self.result[15])
				# added by wei
				self.status[0] = ((opa[15] | opb[15]) & ~self.result[15]) | (opa[15] & opb[15])
				self.status[2] = self.result[15]
				self.status[8] = ((opa[15] & opb[15]) & ~self.result[15]) | ((~opa[15] & ~opb[15]) & self.result[15])

			self.status[1] = (self.result == 0)

			
		# CMP
		if self.ir.double_opcode() == 9:
			wr = 0
			
			
			
		# DADD
		if self.ir.double_opcode() == 10:
			c = Int(1)
			c = 0
			self.result = 0
			self.result[0:4] = self.ops[0:4] + self.opd[0:4]
			# optimize!!!
			if self.result[3] and (self.result[2] or self.result[1]):
			#if self.result[0:4] > 9:
				self.result[0:4] = self.result[0:4] + 6
				c = 1
				
			self.result[4:8] = self.ops[4:8] + self.opd[4:8] + c
			c = 0
			# optimize!!!
			if self.result[7] and (self.result[5] or self.result[6]):
			#if self.result[4:8] > 9:
				self.result[4:8] = self.result[4:8] + 6
				c = 1

			# Word
			if self.ir.double_bw() == 0:
				self.result[8:12] = self.ops[8:12] + self.opd[8:12] + c
				c = 0
				# optimize!!!
				if self.result[11] and (self.result[9] or self.result[10]):
				#if self.result[8:12] > 9:
					self.result[8:12] = self.result[8:12] + 6
					c = 1
				self.result[12:16] = self.ops[12:16] + self.opd[12:16] + c
				c = 0
				# optimize!!!
				if self.result[15] and (self.result[13] or self.result[14]):
				#if self.result[12:16] > 9:
					self.result[12:16] = self.result[12:16] + 6
					c = 1
			
			self.status[0] = c
			self.status[1] = (self.result == 0)

			if self.ir.double_bw() == 1:
				self.status[2] = self.result[7]
			else:
				self.status[2] = self.result[15]

			
		# BIT
		if self.ir.double_opcode() == 11:
			self.result = self.ops & self.opd
			wr = 0

			if self.ir.double_bw() == 1:
				self.result = self.result & 0xFF
				self.status[2] = self.result[7]

			else:
				self.status[2] = self.result[15]
			
			self.status[0] = (self.result != 0)
			self.status[1] = (self.result == 0)
			self.status[8] = 0
			
		# BIC
		if self.ir.double_opcode() == 12:
			self.result = self.opd & ~self.ops
			
		# BIS
		if self.ir.double_opcode() == 13:
			self.result = self.opd | self.ops
			
		# XOR
		if self.ir.double_opcode() == 14:
			self.result = self.opd ^ self.ops

			if self.ir.double_bw() == 1:
				self.result = self.result & 0xFF
				self.status[2] = self.result[7]
				self.status[8] = self.ops[7] & self.opd[7]

			else:
				self.status[2] = self.result[15]
				self.status[8] = self.ops[15] & self.opd[15]
			
			self.status[0] = (self.result != 0)
			self.status[1] = (self.result == 0)

			
		# AND
		if self.ir.double_opcode() == 15:
			self.result = self.opd & self.ops

			if self.ir.double_bw() == 1:
				self.result = self.result & 0xFF
				self.status[2] = self.result[7]

			else:
				self.status[2] = self.result[15]
			
			self.status[0] = (self.result != 0)
			self.status[1] = (self.result == 0)
			self.status[8] = 0
			

		if wr == 1:
			# Indirect
			if self.ir.double_ad() == 1:
				self.mWriteAddrOut = self.memaddrd
				self.mWriteDataOut = self.result
				self.mWriteByte = self.ir.double_bw()
				self.mWriteOpTagOut = 1
				create(tMemWrite)
				create(tCommitDoubleMemWait)
				
			else:
				if self.ir.double_dreg() == 0:
					self.pc = self.result
					create(tCommit)
					
				elif self.ir.double_dreg() == 1:
					self.sp = self.result
					create(tCommit)
					
				elif self.ir.double_dreg() == 2:
					self.status = self.result
					create(tCommit)
					
				else:
					self.rWriteAddrOut = self.ir.double_dreg()
					self.rWriteThreadOut = self.thread
					self.rWriteDataOut = self.result
					self.rWriteOpTagOut = 1
					create(tRegWrite)
					create(tCommitDoubleRegWait)			
		else:
			create(tCommit)
			
			
		

	# ------------------------------------------------------------------------------
	# COMMIT
	# ------------------------------------------------------------------------------
	def commit(self, trigger="tCommit or (tRsMemWriteDone and tCommitSingleMemWait) or (tRsRegWriteDone and tCommitSingleRegWait) or (tRdMemWriteDone and tCommitDoubleMemWait) or (tRdRegWriteDone and tCommitDoubleRegWait)"):
		self.spo = self.sp
		self.sro = self.status
		self.threado = self.thread

		# CALL
		if self.ir.is_single() and self.ir.single_opcode() == 5:
			self.pco = self.ops
		else:
			self.pco = self.pc

		create(tDone)

	# ------------------------------------------------------------------------------
	# END COMMIT
	# ------------------------------------------------------------------------------
	
			
			
   
   
	# ----------------------------------#
	#		REGISTER INTERFACE		    #
	# ----------------------------------#
	def regReadDone(self, trigger="tRegReadDone"):
		self.value = self.rReadDataIn

		if self.rReadOpTagIn == 0:
			create(tRsRegReadDone)

		if self.rReadOpTagIn == 1:
			create(tRdRegReadDone)
			
	def regWriteDone(self, trigger="tRegWriteDone"):
		if self.rWriteOpTagIn == 0:
			create(tRsRegWriteDone)

		if self.rWriteOpTagIn == 1:
			create(tRdRegWriteDone)

		if self.rWriteOpTagIn == 2:
			create(tIncRegWriteDone)
			
	# ----------------------------------#
	#		 MEMORY INTERFACE		    #
	# ----------------------------------#
	def memReadDone(self, trigger="tMemReadDone"):
		self.value = self.mReadDataIn
		
		if self.mReadOpTagIn == 2:
			create(tRsMemReadDone)

		if self.mReadOpTagIn == 4:
			create(tRdMemReadDone)

		if self.mReadOpTagIn == 5:
			create(tRetiSrMemReadDone)

		if self.mReadOpTagIn == 6:
			create(tRetiPcMemReadDone)

		if self.mReadOpTagIn == 7:
			create(tIntMemReadDone)
		
			
	def memWriteDone(self, trigger="tMemWriteDone"):
		if self.mWriteOpTagIn == 0:
			create(tRsMemWriteDone)

		if self.mWriteOpTagIn == 1:
			create(tRdMemWriteDone)

		if self.mWriteOpTagIn == 2:
			create(tIntPCMemWriteDone)
			
		if self.mWriteOpTagIn == 3:
			create(tIntSRMemWriteDone)
			
			
			
			
