/* --- rsLatchSep --- */
function [198:0] rsLatchSep;
input [47:0] _ir;
input _ismems;
input [15:0] _mReadAddrOut;
input _mReadByte;
input [2:0] _mReadOpTagOut;
input [15:0] _memaddrs;
input [15:0] _ops;
input [15:0] _pc;
input [3:0] _rReadAddrOut;
input _rReadOpTagOut;
input [1:0] _rReadThreadOut;
input [3:0] _rWriteAddrOut;
input [15:0] _rWriteDataOut;
input [1:0] _rWriteOpTagOut;
input [1:0] _rWriteThreadOut;
input [15:0] _sp;
input [15:0] _status;
input [1:0] _thread;
input [15:0] _value;

// tokens
reg tRsRegLatch_;

// local storage
reg [15:0] status_;
reg [47:0] ir_;
reg [1:0] rReadThreadOut_;
reg [15:0] memaddrs_;
reg rReadOpTagOut_;
reg [15:0] ops_;
reg [1:0] rWriteOpTagOut_;
reg [1:0] rWriteThreadOut_;
reg [3:0] rWriteAddrOut_;
reg [15:0] value_;
reg [15:0] pc_;
reg [1:0] thread_;
reg [15:0] mReadAddrOut_;
reg mReadByte_;
reg [15:0] rWriteDataOut_;
reg [3:0] rReadAddrOut_;
reg ismems_;
reg [2:0] mReadOpTagOut_;
reg [15:0] sp_;

begin
	// init token outputs
	tRsRegLatch_ = 1'b0;

	// locals init
	status_ = _status;
	rWriteAddrOut_ = _rWriteAddrOut;
	rReadThreadOut_ = _rReadThreadOut;
	memaddrs_ = _memaddrs;
	rReadOpTagOut_ = _rReadOpTagOut;
	ops_ = _ops;
	rWriteOpTagOut_ = _rWriteOpTagOut;
	rWriteThreadOut_ = _rWriteThreadOut;
	ir_ = _ir;
	value_ = _value;
	mReadOpTagOut_ = _mReadOpTagOut;
	pc_ = _pc;
	thread_ = _thread;
	mReadAddrOut_ = _mReadAddrOut;
	rWriteDataOut_ = _rWriteDataOut;
	rReadAddrOut_ = _rReadAddrOut;
	ismems_ = _ismems;
	mReadByte_ = _mReadByte;
	sp_ = _sp;

	// begin functional code
	tRsRegLatch_ = 1'b1;

	rsLatchSep = {tRsRegLatch_, ir_, ismems_, mReadAddrOut_, mReadByte_, mReadOpTagOut_, memaddrs_, ops_, pc_, rReadAddrOut_, rReadOpTagOut_, rReadThreadOut_, rWriteAddrOut_, rWriteDataOut_, rWriteOpTagOut_, rWriteThreadOut_, sp_, status_, thread_, value_};
end
endfunction

/* --- pcIntPushDone --- */
function [85:0] pcIntPushDone;
input [15:0] _mWriteAddrOut;
input _mWriteByte;
input [15:0] _mWriteDataOut;
input [2:0] _mWriteOpTagOut;
input [15:0] _pc;
input [15:0] _sp;
input [15:0] _status;

// tokens
reg tIntSRMemWriteWait_;
reg tMemWrite_;

// local storage
reg [15:0] status_;
reg [2:0] mWriteOpTagOut_;
reg [15:0] mWriteDataOut_;
reg mWriteByte_;
reg [15:0] sp_;
reg [15:0] pc_;
reg [15:0] mWriteAddrOut_;

begin
	// init token outputs
	tIntSRMemWriteWait_ = 1'b0;
	tMemWrite_ = 1'b0;

	// locals init
	status_ = _status;
	mWriteOpTagOut_ = _mWriteOpTagOut;
	mWriteDataOut_ = _mWriteDataOut;
	mWriteByte_ = _mWriteByte;
	sp_ = _sp;
	pc_ = _pc;
	mWriteAddrOut_ = _mWriteAddrOut;

	// begin functional code
	sp_ = (sp_ - 16'd2);
	mWriteAddrOut_ = sp_;
	mWriteDataOut_ = status_;
	mWriteByte_ = 1'd0;
	mWriteOpTagOut_ = 3'd3;
	tMemWrite_ = 1'b1;
	tIntSRMemWriteWait_ = 1'b1;

	pcIntPushDone = {tIntSRMemWriteWait_, tMemWrite_, mWriteAddrOut_, mWriteByte_, mWriteDataOut_, mWriteOpTagOut_, pc_, sp_, status_};
end
endfunction

/* --- rsMemReadDone --- */
function [198:0] rsMemReadDone;
input [47:0] _ir;
input _ismems;
input [15:0] _mReadAddrOut;
input _mReadByte;
input [2:0] _mReadOpTagOut;
input [15:0] _memaddrs;
input [15:0] _ops;
input [15:0] _pc;
input [3:0] _rReadAddrOut;
input _rReadOpTagOut;
input [1:0] _rReadThreadOut;
input [3:0] _rWriteAddrOut;
input [15:0] _rWriteDataOut;
input [1:0] _rWriteOpTagOut;
input [1:0] _rWriteThreadOut;
input [15:0] _sp;
input [15:0] _status;
input [1:0] _thread;
input [15:0] _value;

// tokens
reg tRsRegDone_;

// local storage
reg [15:0] status_;
reg [47:0] ir_;
reg [1:0] rReadThreadOut_;
reg [15:0] memaddrs_;
reg rReadOpTagOut_;
reg [15:0] ops_;
reg [1:0] rWriteOpTagOut_;
reg [1:0] rWriteThreadOut_;
reg [3:0] rWriteAddrOut_;
reg [15:0] value_;
reg [15:0] pc_;
reg [1:0] thread_;
reg [15:0] mReadAddrOut_;
reg mReadByte_;
reg [15:0] rWriteDataOut_;
reg [3:0] rReadAddrOut_;
reg ismems_;
reg [2:0] mReadOpTagOut_;
reg [15:0] sp_;

begin
	// init token outputs
	tRsRegDone_ = 1'b0;

	// locals init
	status_ = _status;
	rWriteAddrOut_ = _rWriteAddrOut;
	rReadThreadOut_ = _rReadThreadOut;
	memaddrs_ = _memaddrs;
	rReadOpTagOut_ = _rReadOpTagOut;
	ops_ = _ops;
	rWriteOpTagOut_ = _rWriteOpTagOut;
	rWriteThreadOut_ = _rWriteThreadOut;
	ir_ = _ir;
	value_ = _value;
	mReadOpTagOut_ = _mReadOpTagOut;
	pc_ = _pc;
	thread_ = _thread;
	mReadAddrOut_ = _mReadAddrOut;
	rWriteDataOut_ = _rWriteDataOut;
	rReadAddrOut_ = _rReadAddrOut;
	ismems_ = _ismems;
	mReadByte_ = _mReadByte;
	sp_ = _sp;

	// begin functional code
	ops_ = value_;
	tRsRegDone_ = 1'b1;

	rsMemReadDone = {tRsRegDone_, ir_, ismems_, mReadAddrOut_, mReadByte_, mReadOpTagOut_, memaddrs_, ops_, pc_, rReadAddrOut_, rReadOpTagOut_, rReadThreadOut_, rWriteAddrOut_, rWriteDataOut_, rWriteOpTagOut_, rWriteThreadOut_, sp_, status_, thread_, value_};
end
endfunction

/* --- rsReady --- */
function [276:0] rsReady;
input [47:0] _ir;
input _ismems;
input [15:0] _mReadAddrOut;
input _mReadByte;
input [2:0] _mReadOpTagOut;
input [15:0] _memaddrs;
input [15:0] _ops;
input [15:0] _pc;
input [3:0] _rReadAddrOut;
input _rReadOpTagOut;
input [1:0] _rReadThreadOut;
input [3:0] _rWriteAddrOut;
input [15:0] _rWriteDataOut;
input [1:0] _rWriteOpTagOut;
input [1:0] _rWriteThreadOut;
input [15:0] _sp;
input [15:0] _status;
input [1:0] _thread;
input [15:0] _value;

// tokens
reg tCommit_;
reg tMemWrite_;
reg tRetiSrMemWait_;
reg tRegWrite_;
reg tCommitSingleRegWait_;
reg tCommitDoubleRegWait_;
reg tRegRead_;
reg tRdRegWait_;
reg tMemRead_;
reg tRdIsInternal_;
reg tCommitSingleMemWait_;

// local storage
reg rReadOpTagOut_;
reg [15:0] mWriteDataOut_;
reg [1:0] rWriteOpTagOut_;
reg [1:0] rWriteThreadOut_;
reg [47:0] ir_;
reg [15:0] mReadAddrOut_;
reg [15:0] result_;
reg [2:0] mWriteOpTagOut_;
reg mReadByte_;
reg [15:0] pc_;
reg [15:0] rWriteDataOut_;
reg ismems_;
reg mWriteByte_;
reg [15:0] status_;
reg [15:0] memaddrs_;
reg [15:0] ops_;
reg [3:0] rWriteAddrOut_;
reg [15:0] opd_;
reg [3:0] rReadAddrOut_;
reg [1:0] rReadThreadOut_;
reg [1:0] thread_;
reg [15:0] sp_;
reg [15:0] value_;
reg [15:0] mWriteAddrOut_;
reg [2:0] mReadOpTagOut_;
reg [4:0] rd;
reg md;
reg c;
reg wr;

begin
	// init token outputs
	tCommit_ = 1'b0;
	tMemWrite_ = 1'b0;
	tRetiSrMemWait_ = 1'b0;
	tRegWrite_ = 1'b0;
	tCommitSingleRegWait_ = 1'b0;
	tCommitDoubleRegWait_ = 1'b0;
	tRegRead_ = 1'b0;
	tRdRegWait_ = 1'b0;
	tMemRead_ = 1'b0;
	tRdIsInternal_ = 1'b0;
	tCommitSingleMemWait_ = 1'b0;

	// locals init
	status_ = _status;
	rWriteAddrOut_ = _rWriteAddrOut;
	rReadThreadOut_ = _rReadThreadOut;
	memaddrs_ = _memaddrs;
	thread_ = _thread;
	ops_ = _ops;
	rWriteOpTagOut_ = _rWriteOpTagOut;
	rReadOpTagOut_ = _rReadOpTagOut;
	ir_ = _ir;
	value_ = _value;
	mReadOpTagOut_ = _mReadOpTagOut;
	pc_ = _pc;
	mReadAddrOut_ = _mReadAddrOut;
	mReadByte_ = _mReadByte;
	rWriteDataOut_ = _rWriteDataOut;
	rReadAddrOut_ = _rReadAddrOut;
	ismems_ = _ismems;
	rWriteThreadOut_ = _rWriteThreadOut;
	sp_ = _sp;

	// begin functional code
	if (Instruction_is_single(ir_)) begin
		wr = 1'd0;
		wr = 1'd1;
		if (Instruction_single_opcode(ir_) == 3'd0) begin
			c = 1'd0;
			c = ops_[0];
			result_ = (ops_ >> 16'd1);
			if (Instruction_single_bw(ir_) == 1'd1) begin
				result_ = (result_ & 16'd127);
				result_ = (result_ | (Status_C(status_) << 16'd7));
			end
			else begin
				result_ = (result_ | (Status_C(status_) << 16'd15));
			end
			status_[0] = c;
			status_[1] = result_ == 16'd0;
			status_[2] = result_[15];
			status_[8] = 1'd0;
		end
		if (Instruction_single_opcode(ir_) == 3'd1) begin
			result_ = ((ops_[7:0] << 8'd8) | (ops_[15:8] >> 8'd8));
		end
		if (Instruction_single_opcode(ir_) == 3'd2) begin
			status_[0] = ops_[0];
			if (Instruction_single_bw(ir_) == 1'd1) begin
				result_ = ((ops_ & 16'd128) | (ops_[7:0] >> 16'd1));
				status_[2] = result_[7];
			end
			else begin
				result_ = ((ops_ & 16'd32768) | (ops_[15:0] >> 16'd1));
				status_[2] = result_[15];
			end
			status_[1] = result_ == 16'd0;
			status_[8] = 1'd0;
		end
		if (Instruction_single_opcode(ir_) == 3'd3) begin
			result_ = ((ops_[7] << 8'd8) | (ops_[7] << 8'd7) | (ops_[7] << 8'd6) | (ops_[7] << 8'd5) | (ops_[7] << 8'd4) | (ops_[7] << 8'd3) | (ops_[7] << 8'd2) | (ops_[7] << 8'd1) | ops_[7:0]);
			status_[0] = result_ != 16'd0;
			status_[1] = result_ == 16'd0;
			status_[2] = result_[15];
			status_[8] = 1'd0;
		end
		if (Instruction_single_opcode(ir_) == 3'd4) begin
			sp_ = (sp_ - 16'd2);
			mWriteAddrOut_ = sp_;
			mWriteDataOut_ = ops_;
			mWriteByte_ = Instruction_single_bw(ir_);
			mWriteOpTagOut_ = 3'd0;
			tMemWrite_ = 1'b1;
			tCommitSingleMemWait_ = 1'b1;
			wr = 1'd0;
		end
		if (Instruction_single_opcode(ir_) == 3'd5) begin
			sp_ = (sp_ - 16'd2);
			mWriteAddrOut_ = sp_;
			mWriteDataOut_ = pc_;
			mWriteByte_ = 1'd0;
			mWriteOpTagOut_ = 3'd0;
			tMemWrite_ = 1'b1;
			tCommitSingleMemWait_ = 1'b1;
			wr = 1'd0;
		end
		if (Instruction_single_opcode(ir_) == 3'd6) begin
			mReadAddrOut_ = sp_;
			mReadByte_ = 1'd0;
			mReadOpTagOut_ = 3'd5;
			tMemRead_ = 1'b1;
			tRetiSrMemWait_ = 1'b1;
			sp_ = (sp_ + 16'd2);
			wr = 1'd0;
		end
		if (wr == 1'd1) begin
			if (ismems_ == 1'd1) begin
				mWriteAddrOut_ = memaddrs_;
				mWriteDataOut_ = result_;
				mWriteByte_ = Instruction_single_bw(ir_);
				mWriteOpTagOut_ = 3'd0;
				tMemWrite_ = 1'b1;
				tCommitSingleMemWait_ = 1'b1;
			end
			else begin
				rWriteAddrOut_ = Instruction_single_dsreg(ir_);
				rWriteThreadOut_ = thread_;
				rWriteDataOut_ = result_;
				rWriteOpTagOut_ = 2'd0;
				tRegWrite_ = 1'b1;
				tCommitSingleRegWait_ = 1'b1;
			end
		end
	end
	else begin
		if ((Instruction_double_opcode(ir_) == 4'd4 & Instruction_double_ad(ir_) == 1'd0)) begin
			if (Instruction_double_dreg(ir_) == 4'd0) begin
				pc_ = ops_;
				tCommit_ = 1'b1;
			end
			else if (Instruction_double_dreg(ir_) == 4'd1) begin
				sp_ = ops_;
				tCommit_ = 1'b1;
			end
			else if (Instruction_double_dreg(ir_) == 4'd2) begin
				status_ = ops_;
				tCommit_ = 1'b1;
			end
			else begin
				rWriteAddrOut_ = Instruction_double_dreg(ir_);
				rWriteThreadOut_ = thread_;
				rWriteDataOut_ = ops_;
				rWriteOpTagOut_ = 2'd1;
				tRegWrite_ = 1'b1;
				tCommitDoubleRegWait_ = 1'b1;
			end
		end
		else begin
			rd = 5'd0;
			md = 1'd0;
			rd = Instruction_double_dreg(ir_);
			md = Instruction_double_ad(ir_);
			if (rd == 5'd0) begin
				opd_ = pc_;
				tRdIsInternal_ = 1'b1;
			end
			else if (rd == 5'd1) begin
				opd_ = sp_;
				tRdIsInternal_ = 1'b1;
			end
			else if (rd == 5'd2) begin
				if (md == 1'd0) begin
					opd_ = status_;
				end
				else begin
					opd_ = 16'd0;
				end
				tRdIsInternal_ = 1'b1;
			end
			else if (rd == 5'd3) begin
				opd_ = 16'd0;
				tRdIsInternal_ = 1'b1;
			end
			else begin
				rReadAddrOut_ = Instruction_double_dreg(ir_);
				rReadThreadOut_ = thread_;
				rReadOpTagOut_ = 1'd1;
				tRegRead_ = 1'b1;
				tRdRegWait_ = 1'b1;
			end
		end
	end

	rsReady = {tCommit_, tCommitDoubleRegWait_, tCommitSingleMemWait_, tCommitSingleRegWait_, tMemRead_, tMemWrite_, tRdIsInternal_, tRdRegWait_, tRegRead_, tRegWrite_, tRetiSrMemWait_, ir_, ismems_, mReadAddrOut_, mReadByte_, mReadOpTagOut_, mWriteAddrOut_, mWriteByte_, mWriteDataOut_, mWriteOpTagOut_, memaddrs_, opd_, ops_, pc_, rReadAddrOut_, rReadOpTagOut_, rReadThreadOut_, rWriteAddrOut_, rWriteDataOut_, rWriteOpTagOut_, rWriteThreadOut_, result_, sp_, status_, thread_, value_};
end
endfunction

/* --- regWriteDone --- */
function [4:0] regWriteDone;
input [1:0] _rWriteOpTagIn;

// tokens
reg tRsRegWriteDone_;
reg tIncRegWriteDone_;
reg tRdRegWriteDone_;

// local storage
reg [1:0] rWriteOpTagIn_;

begin
	// init token outputs
	tRsRegWriteDone_ = 1'b0;
	tIncRegWriteDone_ = 1'b0;
	tRdRegWriteDone_ = 1'b0;

	// locals init
	rWriteOpTagIn_ = _rWriteOpTagIn;

	// begin functional code
	if (rWriteOpTagIn_ == 2'd0) begin
		tRsRegWriteDone_ = 1'b1;
	end
	if (rWriteOpTagIn_ == 2'd1) begin
		tRdRegWriteDone_ = 1'b1;
	end
	if (rWriteOpTagIn_ == 2'd2) begin
		tIncRegWriteDone_ = 1'b1;
	end

	regWriteDone = {tIncRegWriteDone_, tRdRegWriteDone_, tRsRegWriteDone_, rWriteOpTagIn_};
end
endfunction

/* --- decode --- */
function [124:0] decode;
input [47:0] _ir;
input [15:0] _pc;
input [15:0] _sp;
input [15:0] _status;
input [1:0] _thread;

// tokens
reg tRsRegWait_;
reg tRegRead_;
reg tCommit_;
reg tRsIsInternal_;

// local storage
reg [15:0] status_;
reg [47:0] ir_;
reg [1:0] rReadThreadOut_;
reg [1:0] thread_;
reg rReadOpTagOut_;
reg [15:0] ops_;
reg [15:0] sp_;
reg [15:0] pc_;
reg [3:0] rReadAddrOut_;
reg jmp;
reg [3:0] rs;
reg [1:0] ms;
reg [15:0] offset;

begin
	// init token outputs
	tRsRegWait_ = 1'b0;
	tRegRead_ = 1'b0;
	tCommit_ = 1'b0;
	tRsIsInternal_ = 1'b0;

	// locals init
	status_ = _status;
	sp_ = _sp;
	thread_ = _thread;
	ir_ = _ir;
	pc_ = _pc;

	// begin functional code
	offset = 16'd0;
	offset = 16'd0;
	if (Instruction_is_jump(ir_)) begin
		jmp = 1'd0;
		jmp = 1'd0;
		if (Instruction_jump_c(ir_) == 3'd0) begin
			if (Status_Z(status_) == 1'd0) begin
				jmp = 1'd1;
			end
		end
		if (Instruction_jump_c(ir_) == 3'd1) begin
			if (Status_Z(status_) == 1'd1) begin
				jmp = 1'd1;
			end
		end
		if (Instruction_jump_c(ir_) == 3'd2) begin
			if (Status_C(status_) == 1'd0) begin
				jmp = 1'd1;
			end
		end
		if (Instruction_jump_c(ir_) == 3'd3) begin
			if (Status_C(status_) == 1'd1) begin
				jmp = 1'd1;
			end
		end
		if (Instruction_jump_c(ir_) == 3'd4) begin
			if (Status_N(status_) == 1'd1) begin
				jmp = 1'd1;
			end
		end
		if (Instruction_jump_c(ir_) == 3'd5) begin
			if (Status_N(status_) == Status_V(status_)) begin
				jmp = 1'd1;
			end
		end
		if (Instruction_jump_c(ir_) == 3'd6) begin
			if (Status_N(status_) != Status_V(status_)) begin
				jmp = 1'd1;
			end
		end
		if (Instruction_jump_c(ir_) == 3'd7) begin
			jmp = 1'd1;
		end
		if (jmp) begin
			offset = Instruction_jump_offset(ir_);
		end
		tCommit_ = 1'b1;
	end
	else begin
		rs = 4'd0;
		ms = 2'd0;
		if (Instruction_is_single(ir_)) begin
			rs = Instruction_single_dsreg(ir_);
			ms = Instruction_single_ad(ir_);
		end
		else if (Instruction_is_double(ir_)) begin
			rs = Instruction_double_sreg(ir_);
			ms = Instruction_double_as(ir_);
		end
		if (rs == 4'd0) begin
			ops_ = pc_;
			tRsIsInternal_ = 1'b1;
		end
		else if (rs == 4'd1) begin
			ops_ = sp_;
			tRsIsInternal_ = 1'b1;
		end
		else if ((rs == 4'd2 | rs == 4'd3)) begin
			if (rs == 4'd2) begin
				if (ms == 2'd0) begin
					ops_ = status_;
				end
				else if (ms == 2'd1) begin
					ops_ = 16'd0;
				end
				else if (ms == 2'd2) begin
					ops_ = 16'd4;
				end
				else if (ms == 2'd3) begin
					ops_ = 16'd8;
				end
				tRsIsInternal_ = 1'b1;
			end
			else if (rs == 4'd3) begin
				if (ms == 2'd0) begin
					ops_ = 16'd0;
				end
				if (ms == 2'd1) begin
					ops_ = 16'd1;
				end
				if (ms == 2'd2) begin
					ops_ = 16'd2;
				end
				if (ms == 2'd3) begin
					ops_ = 16'd65535;
				end
				tRsIsInternal_ = 1'b1;
			end
		end
		else begin
			rReadAddrOut_ = rs;
			rReadThreadOut_ = thread_;
			rReadOpTagOut_ = 1'd0;
			tRegRead_ = 1'b1;
			tRsRegWait_ = 1'b1;
		end
	end
	pc_ = ((pc_ + offset) + 16'd2);

	decode = {tCommit_, tRegRead_, tRsIsInternal_, tRsRegWait_, ir_, ops_, pc_, rReadAddrOut_, rReadOpTagOut_, rReadThreadOut_, sp_, status_, thread_};
end
endfunction

/* --- rsGetRegDone --- */
function [202:0] rsGetRegDone;
input [47:0] _ir;
input [15:0] _ops;
input [15:0] _pc;
input [3:0] _rReadAddrOut;
input _rReadOpTagOut;
input [1:0] _rReadThreadOut;
input [15:0] _sp;
input [15:0] _status;
input [1:0] _thread;
input [15:0] _value;

// tokens
reg tRsMemWait_;
reg tRegWrite_;
reg tRsRegDone_;
reg tNoInc_;
reg tMemRead_;

// local storage
reg [15:0] status_;
reg [15:0] sp_;
reg [1:0] rReadThreadOut_;
reg [15:0] memaddrs_;
reg rReadOpTagOut_;
reg [15:0] ops_;
reg [1:0] rWriteOpTagOut_;
reg [1:0] rWriteThreadOut_;
reg [3:0] rWriteAddrOut_;
reg [15:0] value_;
reg [2:0] mReadOpTagOut_;
reg [15:0] pc_;
reg [1:0] thread_;
reg [15:0] mReadAddrOut_;
reg [15:0] rWriteDataOut_;
reg [3:0] rReadAddrOut_;
reg ismems_;
reg mReadByte_;
reg [47:0] ir_;
reg [1:0] add;
reg bw;
reg [1:0] ms;
reg [3:0] rs;

begin
	// init token outputs
	tRsMemWait_ = 1'b0;
	tRegWrite_ = 1'b0;
	tRsRegDone_ = 1'b0;
	tNoInc_ = 1'b0;
	tMemRead_ = 1'b0;

	// locals init
	status_ = _status;
	sp_ = _sp;
	rReadThreadOut_ = _rReadThreadOut;
	thread_ = _thread;
	rReadOpTagOut_ = _rReadOpTagOut;
	ops_ = _ops;
	ir_ = _ir;
	value_ = _value;
	pc_ = _pc;
	rReadAddrOut_ = _rReadAddrOut;

	// begin functional code
	rs = 4'd0;
	ms = 2'd0;
	bw = 1'd0;
	if (Instruction_is_single(ir_)) begin
		rs = Instruction_single_dsreg(ir_);
		ms = Instruction_single_ad(ir_);
		bw = Instruction_single_bw(ir_);
	end
	else if (Instruction_is_double(ir_)) begin
		rs = Instruction_double_sreg(ir_);
		ms = Instruction_double_as(ir_);
		bw = Instruction_double_bw(ir_);
	end
	ismems_ = 1'd0;
	if ((ms == 2'd0 | rs == 4'd3 | (rs == 4'd2 & ms != 2'd1))) begin
		tRsRegDone_ = 1'b1;
	end
	else begin
		memaddrs_ = ops_;
		ismems_ = 1'd1;
		if ((ms == 2'd1 | (ms == 2'd3 & rs == 4'd0))) begin
			pc_ = (pc_ + 16'd2);
		end
		if (ms == 2'd1) begin
			if (rs == 4'd2) begin
				memaddrs_ = Instruction_extension1(ir_);
			end
			else begin
				memaddrs_ = (memaddrs_ + Instruction_extension1(ir_));
			end
		end
		if ((ms == 2'd3 & rs == 4'd0)) begin
			ops_ = Instruction_extension1(ir_);
			tRsRegDone_ = 1'b1;
		end
		else begin
			mReadAddrOut_ = memaddrs_;
			mReadOpTagOut_ = 3'd2;
			mReadByte_ = bw;
			tRsMemWait_ = 1'b1;
			tMemRead_ = 1'b1;
			if (ms == 2'd3) begin
				if (rs == 4'd1) begin
					sp_ = (sp_ + 16'd2);
					tNoInc_ = 1'b1;
				end
				else if ((rs[2] | rs[3])) begin
					rWriteOpTagOut_ = 2'd2;
					rWriteAddrOut_ = rs;
					rWriteThreadOut_ = thread_;
					add = 2'd0;
					add = 2'd2;
					if ((bw & rs != 4'd0 & rs != 4'd1)) begin
						add = 2'd1;
					end
					rWriteDataOut_ = (ops_ + add);
					tRegWrite_ = 1'b1;
				end
			end
			else begin
				tNoInc_ = 1'b1;
			end
		end
	end

	rsGetRegDone = {tMemRead_, tNoInc_, tRegWrite_, tRsMemWait_, tRsRegDone_, ir_, ismems_, mReadAddrOut_, mReadByte_, mReadOpTagOut_, memaddrs_, ops_, pc_, rReadAddrOut_, rReadOpTagOut_, rReadThreadOut_, rWriteAddrOut_, rWriteDataOut_, rWriteOpTagOut_, rWriteThreadOut_, sp_, status_, thread_, value_};
end
endfunction

/* --- memReadDone --- */
function [39:0] memReadDone;
input [15:0] _mReadDataIn;
input [2:0] _mReadOpTagIn;

// tokens
reg tRsMemReadDone_;
reg tRdMemReadDone_;
reg tRetiSrMemReadDone_;
reg tRetiPcMemReadDone_;
reg tIntMemReadDone_;

// local storage
reg [15:0] mReadDataIn_;
reg [2:0] mReadOpTagIn_;
reg [15:0] value_;

begin
	// init token outputs
	tRsMemReadDone_ = 1'b0;
	tRdMemReadDone_ = 1'b0;
	tRetiSrMemReadDone_ = 1'b0;
	tRetiPcMemReadDone_ = 1'b0;
	tIntMemReadDone_ = 1'b0;

	// locals init
	mReadDataIn_ = _mReadDataIn;
	mReadOpTagIn_ = _mReadOpTagIn;

	// begin functional code
	value_ = mReadDataIn_;
	if (mReadOpTagIn_ == 3'd2) begin
		tRsMemReadDone_ = 1'b1;
	end
	if (mReadOpTagIn_ == 3'd4) begin
		tRdMemReadDone_ = 1'b1;
	end
	if (mReadOpTagIn_ == 3'd5) begin
		tRetiSrMemReadDone_ = 1'b1;
	end
	if (mReadOpTagIn_ == 3'd6) begin
		tRetiPcMemReadDone_ = 1'b1;
	end
	if (mReadOpTagIn_ == 3'd7) begin
		tIntMemReadDone_ = 1'b1;
	end

	memReadDone = {tIntMemReadDone_, tRdMemReadDone_, tRetiPcMemReadDone_, tRetiSrMemReadDone_, tRsMemReadDone_, mReadDataIn_, mReadOpTagIn_, value_};
end
endfunction

/* --- rsRegDone --- */
function [137:0] rsRegDone;
input [47:0] _ir;
input [15:0] _ops;
input [15:0] _pc;
input [3:0] _rReadAddrOut;
input _rReadOpTagOut;
input [1:0] _rReadThreadOut;
input [15:0] _sp;
input [15:0] _status;
input [1:0] _thread;
input [15:0] _value;

// tokens
reg tRsRegIsDone_;

// local storage
reg [15:0] status_;
reg [47:0] ir_;
reg [1:0] rReadThreadOut_;
reg [1:0] thread_;
reg rReadOpTagOut_;
reg [15:0] ops_;
reg [15:0] sp_;
reg [15:0] pc_;
reg [15:0] value_;
reg [3:0] rReadAddrOut_;

begin
	// init token outputs
	tRsRegIsDone_ = 1'b0;

	// locals init
	status_ = _status;
	sp_ = _sp;
	rReadThreadOut_ = _rReadThreadOut;
	thread_ = _thread;
	rReadOpTagOut_ = _rReadOpTagOut;
	ops_ = _ops;
	ir_ = _ir;
	value_ = _value;
	pc_ = _pc;
	rReadAddrOut_ = _rReadAddrOut;

	// begin functional code
	ops_ = value_;
	tRsRegIsDone_ = 1'b1;

	rsRegDone = {tRsRegIsDone_, ir_, ops_, pc_, rReadAddrOut_, rReadOpTagOut_, rReadThreadOut_, sp_, status_, thread_, value_};
end
endfunction

/* --- rdReady --- */
function [226:0] rdReady;
input [47:0] _ir;
input [15:0] _mWriteAddrOut;
input _mWriteByte;
input [15:0] _mWriteDataOut;
input [2:0] _mWriteOpTagOut;
input [15:0] _memaddrd;
input [15:0] _opd;
input [15:0] _ops;
input [15:0] _pc;
input [3:0] _rWriteAddrOut;
input [15:0] _rWriteDataOut;
input [1:0] _rWriteOpTagOut;
input [1:0] _rWriteThreadOut;
input [15:0] _sp;
input [15:0] _status;
input [1:0] _thread;

// tokens
reg tCommit_;
reg tRegWrite_;
reg tCommitDoubleMemWait_;
reg tMemWrite_;
reg tCommitDoubleRegWait_;

// local storage
reg [15:0] status_;
reg [2:0] mWriteOpTagOut_;
reg [1:0] rWriteThreadOut_;
reg [15:0] sp_;
reg [1:0] thread_;
reg [15:0] mWriteDataOut_;
reg [1:0] rWriteOpTagOut_;
reg [15:0] ops_;
reg [3:0] rWriteAddrOut_;
reg [15:0] opd_;
reg [15:0] pc_;
reg [15:0] memaddrd_;
reg [15:0] rWriteDataOut_;
reg [15:0] result_;
reg [15:0] mWriteAddrOut_;
reg mWriteByte_;
reg [47:0] ir_;
reg c;
reg sub;
reg add;
reg wr;
reg [15:0] opb;
reg [15:0] opa;

begin
	// init token outputs
	tCommit_ = 1'b0;
	tRegWrite_ = 1'b0;
	tCommitDoubleMemWait_ = 1'b0;
	tMemWrite_ = 1'b0;
	tCommitDoubleRegWait_ = 1'b0;

	// locals init
	status_ = _status;
	mWriteOpTagOut_ = _mWriteOpTagOut;
	rWriteAddrOut_ = _rWriteAddrOut;
	mWriteDataOut_ = _mWriteDataOut;
	thread_ = _thread;
	ops_ = _ops;
	rWriteOpTagOut_ = _rWriteOpTagOut;
	rWriteThreadOut_ = _rWriteThreadOut;
	ir_ = _ir;
	opd_ = _opd;
	pc_ = _pc;
	memaddrd_ = _memaddrd;
	rWriteDataOut_ = _rWriteDataOut;
	mWriteAddrOut_ = _mWriteAddrOut;
	mWriteByte_ = _mWriteByte;
	sp_ = _sp;

	// begin functional code
	wr = 1'd0;
	wr = 1'd1;
	opa = 16'd0;
	opb = 16'd0;
	add = 1'd0;
	add = 1'd0;
	sub = 1'd0;
	sub = 1'd0;
	c = 1'd0;
	c = 1'd0;
	if (Instruction_double_opcode(ir_) == 4'd4) begin
		result_ = ops_;
	end
	opa = ops_;
	opb = opd_;
	if (Instruction_double_opcode(ir_) == 4'd5) begin
		add = 1'd1;
	end
	if (Instruction_double_opcode(ir_) == 4'd6) begin
		add = 1'd1;
		c = Status_C(status_);
	end
	if (Instruction_double_opcode(ir_) == 4'd7) begin
		sub = 1'd1;
		c = Status_C(status_);
	end
	if ((Instruction_double_opcode(ir_) == 4'd8 | Instruction_double_opcode(ir_) == 4'd9)) begin
		sub = 1'd1;
		c = 1'd1;
	end
	if ((add | sub)) begin
		if (sub) begin
			opa = ~opa;
		end
		result_ = ((opa + opb) + c);
	end
	if ((Instruction_double_opcode(ir_) == 4'd5 | Instruction_double_opcode(ir_) == 4'd6 | Instruction_double_opcode(ir_) == 4'd7 | Instruction_double_opcode(ir_) == 4'd8 | Instruction_double_opcode(ir_) == 4'd9)) begin
		if (Instruction_double_bw(ir_) == 1'd1) begin
			status_[0] = result_[8];
			status_[2] = result_[7];
			status_[8] = (((opa[7] & opb[7]) & ~result_[7]) | ((~opa[7] & ~opb[7]) & result_[7]));
			result_ = (result_ & 16'd255);
		end
		else begin
			status_[0] = ((opa[15] | opb[15]) & ~result_[15]);
			status_[0] = (((opa[15] | opb[15]) & ~result_[15]) | (opa[15] & opb[15]));
			status_[2] = result_[15];
			status_[8] = (((opa[15] & opb[15]) & ~result_[15]) | ((~opa[15] & ~opb[15]) & result_[15]));
		end
		status_[1] = result_ == 16'd0;
	end
	if (Instruction_double_opcode(ir_) == 4'd9) begin
		wr = 1'd0;
	end
	if (Instruction_double_opcode(ir_) == 4'd10) begin
		c = 1'd0;
		c = 1'd0;
		result_ = 16'd0;
		result_[3:0] = (ops_[3:0] + opd_[3:0]);
		if ((result_[3] & (result_[2] | result_[1]))) begin
			result_[3:0] = (result_[3:0] + 4'd6);
			c = 1'd1;
		end
		result_[7:4] = ((ops_[7:4] + opd_[7:4]) + c);
		c = 1'd0;
		if ((result_[7] & (result_[5] | result_[6]))) begin
			result_[7:4] = (result_[7:4] + 4'd6);
			c = 1'd1;
		end
		if (Instruction_double_bw(ir_) == 1'd0) begin
			result_[11:8] = ((ops_[11:8] + opd_[11:8]) + c);
			c = 1'd0;
			if ((result_[11] & (result_[9] | result_[10]))) begin
				result_[11:8] = (result_[11:8] + 4'd6);
				c = 1'd1;
			end
			result_[15:12] = ((ops_[15:12] + opd_[15:12]) + c);
			c = 1'd0;
			if ((result_[15] & (result_[13] | result_[14]))) begin
				result_[15:12] = (result_[15:12] + 4'd6);
				c = 1'd1;
			end
		end
		status_[0] = c;
		status_[1] = result_ == 16'd0;
		if (Instruction_double_bw(ir_) == 1'd1) begin
			status_[2] = result_[7];
		end
		else begin
			status_[2] = result_[15];
		end
	end
	if (Instruction_double_opcode(ir_) == 4'd11) begin
		result_ = (ops_ & opd_);
		wr = 1'd0;
		if (Instruction_double_bw(ir_) == 1'd1) begin
			result_ = (result_ & 16'd255);
			status_[2] = result_[7];
		end
		else begin
			status_[2] = result_[15];
		end
		status_[0] = result_ != 16'd0;
		status_[1] = result_ == 16'd0;
		status_[8] = 1'd0;
	end
	if (Instruction_double_opcode(ir_) == 4'd12) begin
		result_ = (opd_ & ~ops_);
	end
	if (Instruction_double_opcode(ir_) == 4'd13) begin
		result_ = (opd_ | ops_);
	end
	if (Instruction_double_opcode(ir_) == 4'd14) begin
		result_ = (opd_ ^ ops_);
		if (Instruction_double_bw(ir_) == 1'd1) begin
			result_ = (result_ & 16'd255);
			status_[2] = result_[7];
			status_[8] = (ops_[7] & opd_[7]);
		end
		else begin
			status_[2] = result_[15];
			status_[8] = (ops_[15] & opd_[15]);
		end
		status_[0] = result_ != 16'd0;
		status_[1] = result_ == 16'd0;
	end
	if (Instruction_double_opcode(ir_) == 4'd15) begin
		result_ = (opd_ & ops_);
		if (Instruction_double_bw(ir_) == 1'd1) begin
			result_ = (result_ & 16'd255);
			status_[2] = result_[7];
		end
		else begin
			status_[2] = result_[15];
		end
		status_[0] = result_ != 16'd0;
		status_[1] = result_ == 16'd0;
		status_[8] = 1'd0;
	end
	if (wr == 1'd1) begin
		if (Instruction_double_ad(ir_) == 1'd1) begin
			mWriteAddrOut_ = memaddrd_;
			mWriteDataOut_ = result_;
			mWriteByte_ = Instruction_double_bw(ir_);
			mWriteOpTagOut_ = 3'd1;
			tMemWrite_ = 1'b1;
			tCommitDoubleMemWait_ = 1'b1;
		end
		else begin
			if (Instruction_double_dreg(ir_) == 4'd0) begin
				pc_ = result_;
				tCommit_ = 1'b1;
			end
			else if (Instruction_double_dreg(ir_) == 4'd1) begin
				sp_ = result_;
				tCommit_ = 1'b1;
			end
			else if (Instruction_double_dreg(ir_) == 4'd2) begin
				status_ = result_;
				tCommit_ = 1'b1;
			end
			else begin
				rWriteAddrOut_ = Instruction_double_dreg(ir_);
				rWriteThreadOut_ = thread_;
				rWriteDataOut_ = result_;
				rWriteOpTagOut_ = 2'd1;
				tRegWrite_ = 1'b1;
				tCommitDoubleRegWait_ = 1'b1;
			end
		end
	end
	else begin
		tCommit_ = 1'b1;
	end

	rdReady = {tCommit_, tCommitDoubleMemWait_, tCommitDoubleRegWait_, tMemWrite_, tRegWrite_, ir_, mWriteAddrOut_, mWriteByte_, mWriteDataOut_, mWriteOpTagOut_, memaddrd_, opd_, ops_, pc_, rWriteAddrOut_, rWriteDataOut_, rWriteOpTagOut_, rWriteThreadOut_, result_, sp_, status_, thread_};
end
endfunction

/* --- memWriteDone --- */
function [6:0] memWriteDone;
input [2:0] _mWriteOpTagIn;

// tokens
reg tRdMemWriteDone_;
reg tIntSRMemWriteDone_;
reg tIntPCMemWriteDone_;
reg tRsMemWriteDone_;

// local storage
reg [2:0] mWriteOpTagIn_;

begin
	// init token outputs
	tRdMemWriteDone_ = 1'b0;
	tIntSRMemWriteDone_ = 1'b0;
	tIntPCMemWriteDone_ = 1'b0;
	tRsMemWriteDone_ = 1'b0;

	// locals init
	mWriteOpTagIn_ = _mWriteOpTagIn;

	// begin functional code
	if (mWriteOpTagIn_ == 3'd0) begin
		tRsMemWriteDone_ = 1'b1;
	end
	if (mWriteOpTagIn_ == 3'd1) begin
		tRdMemWriteDone_ = 1'b1;
	end
	if (mWriteOpTagIn_ == 3'd2) begin
		tIntPCMemWriteDone_ = 1'b1;
	end
	if (mWriteOpTagIn_ == 3'd3) begin
		tIntSRMemWriteDone_ = 1'b1;
	end

	memWriteDone = {tIntPCMemWriteDone_, tIntSRMemWriteDone_, tRdMemWriteDone_, tRsMemWriteDone_, mWriteOpTagIn_};
end
endfunction

/* --- regReadDone --- */
function [34:0] regReadDone;
input [15:0] _rReadDataIn;
input _rReadOpTagIn;

// tokens
reg tRdRegReadDone_;
reg tRsRegReadDone_;

// local storage
reg [15:0] rReadDataIn_;
reg rReadOpTagIn_;
reg [15:0] value_;

begin
	// init token outputs
	tRdRegReadDone_ = 1'b0;
	tRsRegReadDone_ = 1'b0;

	// locals init
	rReadDataIn_ = _rReadDataIn;
	rReadOpTagIn_ = _rReadOpTagIn;

	// begin functional code
	value_ = rReadDataIn_;
	if (rReadOpTagIn_ == 1'd0) begin
		tRsRegReadDone_ = 1'b1;
	end
	if (rReadOpTagIn_ == 1'd1) begin
		tRdRegReadDone_ = 1'b1;
	end

	regReadDone = {tRdRegReadDone_, tRsRegReadDone_, rReadDataIn_, rReadOpTagIn_, value_};
end
endfunction

/* --- interrupt --- */
function [89:0] interrupt;
input [15:0] _pc;
input [15:0] _sp;
input [15:0] _sr;
input [3:0] _vector;

// tokens
reg tIntMemReadWait_;
reg tMemRead_;

// local storage
reg [15:0] status_;
reg [15:0] sr_;
reg mReadByte_;
reg [15:0] sp_;
reg [15:0] pc_;
reg [3:0] vector_;
reg [15:0] mReadAddrOut_;
reg [2:0] mReadOpTagOut_;

begin
	// init token outputs
	tIntMemReadWait_ = 1'b0;
	tMemRead_ = 1'b0;

	// locals init
	sr_ = _sr;
	vector_ = _vector;
	sp_ = _sp;
	pc_ = _pc;

	// begin functional code
	status_ = sr_;
	mReadAddrOut_ = ((vector_ << 8'd1) + 16'd224);
	mReadAddrOut_ = (mReadAddrOut_ | 16'd65280);
	mReadByte_ = 1'd0;
	mReadOpTagOut_ = 3'd7;
	tMemRead_ = 1'b1;
	tIntMemReadWait_ = 1'b1;

	interrupt = {tIntMemReadWait_, tMemRead_, mReadAddrOut_, mReadByte_, mReadOpTagOut_, pc_, sp_, sr_, status_, vector_};
end
endfunction

/* --- cycle --- */
function [83:0] cycle;
input [15:0] _pc;
input [15:0] _sp;
input [15:0] _sr;
input [1:0] _thread;

// tokens
reg tRomRead_;
reg tFetch_;

// local storage
reg [15:0] status_;
reg [1:0] thread_;
reg [15:0] sr_;
reg [15:0] sp_;
reg [15:0] pc_;
reg [15:0] romReadAddrOut_;

begin
	// init token outputs
	tRomRead_ = 1'b0;
	tFetch_ = 1'b0;

	// locals init
	pc_ = _pc;
	sp_ = _sp;
	thread_ = _thread;
	sr_ = _sr;

	// begin functional code
	romReadAddrOut_ = pc_;
	tRomRead_ = 1'b1;
	tFetch_ = 1'b1;
	status_ = sr_;

	cycle = {tFetch_, tRomRead_, pc_, romReadAddrOut_, sp_, sr_, status_, thread_};
end
endfunction

/* --- rdGetRegDone --- */
function [244:0] rdGetRegDone;
input [47:0] _ir;
input [15:0] _mReadAddrOut;
input _mReadByte;
input [2:0] _mReadOpTagOut;
input [15:0] _mWriteAddrOut;
input _mWriteByte;
input [15:0] _mWriteDataOut;
input [2:0] _mWriteOpTagOut;
input [15:0] _opd;
input [15:0] _ops;
input [15:0] _pc;
input [3:0] _rWriteAddrOut;
input [15:0] _rWriteDataOut;
input [1:0] _rWriteOpTagOut;
input [1:0] _rWriteThreadOut;
input [15:0] _sp;
input [15:0] _status;
input [1:0] _thread;
input [15:0] _value;

// tokens
reg tRdMemWait_;
reg tRdRegDone_;
reg tMemRead_;

// local storage
reg [15:0] status_;
reg [2:0] mWriteOpTagOut_;
reg [47:0] ir_;
reg [15:0] mWriteDataOut_;
reg [1:0] thread_;
reg [15:0] ops_;
reg [1:0] rWriteOpTagOut_;
reg [1:0] rWriteThreadOut_;
reg [3:0] rWriteAddrOut_;
reg [15:0] value_;
reg [2:0] mReadOpTagOut_;
reg [15:0] opd_;
reg [15:0] pc_;
reg [15:0] memaddrd_;
reg [15:0] mReadAddrOut_;
reg [15:0] rWriteDataOut_;
reg [15:0] mWriteAddrOut_;
reg mReadByte_;
reg mWriteByte_;
reg [15:0] sp_;
reg [15:0] ext;
reg [1:0] ms;
reg [3:0] rs;

begin
	// init token outputs
	tRdMemWait_ = 1'b0;
	tRdRegDone_ = 1'b0;
	tMemRead_ = 1'b0;

	// locals init
	status_ = _status;
	mWriteOpTagOut_ = _mWriteOpTagOut;
	rWriteAddrOut_ = _rWriteAddrOut;
	ops_ = _ops;
	thread_ = _thread;
	mWriteDataOut_ = _mWriteDataOut;
	rWriteOpTagOut_ = _rWriteOpTagOut;
	rWriteThreadOut_ = _rWriteThreadOut;
	ir_ = _ir;
	value_ = _value;
	mReadOpTagOut_ = _mReadOpTagOut;
	opd_ = _opd;
	pc_ = _pc;
	mReadAddrOut_ = _mReadAddrOut;
	rWriteDataOut_ = _rWriteDataOut;
	mWriteAddrOut_ = _mWriteAddrOut;
	mReadByte_ = _mReadByte;
	mWriteByte_ = _mWriteByte;
	sp_ = _sp;

	// begin functional code
	if (Instruction_double_ad(ir_) == 1'd0) begin
		tRdRegDone_ = 1'b1;
	end
	else begin
		memaddrd_ = opd_;
		ms = 2'd0;
		ms = Instruction_double_as(ir_);
		rs = 4'd0;
		rs = Instruction_double_sreg(ir_);
		ext = 16'd0;
		if (((ms == 2'd1 & rs == 4'd0) | (ms == 2'd1 & rs == 4'd1) | (ms == 2'd1 & rs > 4'd3) | (ms == 2'd1 & rs == 4'd2) | (ms == 2'd3 & rs == 4'd0))) begin
			ext = Instruction_extension2(ir_);
		end
		else begin
			ext = Instruction_extension1(ir_);
		end
		memaddrd_ = (memaddrd_ + ext);
		pc_ = (pc_ + 16'd2);
		if (Instruction_double_opcode(ir_) == 4'd4) begin
			tRdRegDone_ = 1'b1;
		end
		else begin
			mReadAddrOut_ = memaddrd_;
			mReadByte_ = Instruction_double_bw(ir_);
			mReadOpTagOut_ = 3'd4;
			tMemRead_ = 1'b1;
			tRdMemWait_ = 1'b1;
		end
	end

	rdGetRegDone = {tMemRead_, tRdMemWait_, tRdRegDone_, ir_, mReadAddrOut_, mReadByte_, mReadOpTagOut_, mWriteAddrOut_, mWriteByte_, mWriteDataOut_, mWriteOpTagOut_, memaddrd_, opd_, ops_, pc_, rWriteAddrOut_, rWriteDataOut_, rWriteOpTagOut_, rWriteThreadOut_, sp_, status_, thread_, value_};
end
endfunction

/* --- intDone --- */
function [48:0] intDone;
input [15:0] _pc;
input [15:0] _sp;
input [15:0] _status;

// tokens
reg tCommit_;

// local storage
reg [15:0] status_;
reg [15:0] pc_;
reg [15:0] sp_;

begin
	// init token outputs
	tCommit_ = 1'b0;

	// locals init
	status_ = _status;
	pc_ = _pc;
	sp_ = _sp;

	// begin functional code
	tCommit_ = 1'b1;

	intDone = {tCommit_, pc_, sp_, status_};
end
endfunction

/* --- retiPcPopDone --- */
function [130:0] retiPcPopDone;
input [47:0] _ir;
input [15:0] _ops;
input [15:0] _pc;
input [15:0] _sp;
input [15:0] _status;
input [1:0] _thread;
input [15:0] _value;

// tokens
reg tCommit_;

// local storage
reg [15:0] status_;
reg [47:0] ir_;
reg [1:0] thread_;
reg [15:0] ops_;
reg [15:0] sp_;
reg [15:0] value_;
reg [15:0] pc_;

begin
	// init token outputs
	tCommit_ = 1'b0;

	// locals init
	status_ = _status;
	sp_ = _sp;
	thread_ = _thread;
	ops_ = _ops;
	ir_ = _ir;
	value_ = _value;
	pc_ = _pc;

	// begin functional code
	pc_ = value_;
	tCommit_ = 1'b1;

	retiPcPopDone = {tCommit_, ir_, ops_, pc_, sp_, status_, thread_, value_};
end
endfunction

/* --- rdMemReadDone --- */
function [222:0] rdMemReadDone;
input [47:0] _ir;
input [15:0] _mWriteAddrOut;
input _mWriteByte;
input [15:0] _mWriteDataOut;
input [2:0] _mWriteOpTagOut;
input [15:0] _memaddrd;
input [15:0] _opd;
input [15:0] _ops;
input [15:0] _pc;
input [3:0] _rWriteAddrOut;
input [15:0] _rWriteDataOut;
input [1:0] _rWriteOpTagOut;
input [1:0] _rWriteThreadOut;
input [15:0] _sp;
input [15:0] _status;
input [1:0] _thread;
input [15:0] _value;

// tokens
reg tRdRegDone_;

// local storage
reg [15:0] status_;
reg [2:0] mWriteOpTagOut_;
reg [47:0] ir_;
reg [1:0] thread_;
reg [15:0] mWriteDataOut_;
reg [1:0] rWriteOpTagOut_;
reg [15:0] ops_;
reg [3:0] rWriteAddrOut_;
reg [15:0] value_;
reg [15:0] opd_;
reg [15:0] pc_;
reg [15:0] memaddrd_;
reg [15:0] rWriteDataOut_;
reg [15:0] mWriteAddrOut_;
reg [1:0] rWriteThreadOut_;
reg mWriteByte_;
reg [15:0] sp_;

begin
	// init token outputs
	tRdRegDone_ = 1'b0;

	// locals init
	status_ = _status;
	mWriteOpTagOut_ = _mWriteOpTagOut;
	rWriteAddrOut_ = _rWriteAddrOut;
	ops_ = _ops;
	thread_ = _thread;
	mWriteDataOut_ = _mWriteDataOut;
	rWriteOpTagOut_ = _rWriteOpTagOut;
	rWriteThreadOut_ = _rWriteThreadOut;
	ir_ = _ir;
	value_ = _value;
	opd_ = _opd;
	pc_ = _pc;
	memaddrd_ = _memaddrd;
	rWriteDataOut_ = _rWriteDataOut;
	mWriteAddrOut_ = _mWriteAddrOut;
	mWriteByte_ = _mWriteByte;
	sp_ = _sp;

	// begin functional code
	opd_ = value_;
	tRdRegDone_ = 1'b1;

	rdMemReadDone = {tRdRegDone_, ir_, mWriteAddrOut_, mWriteByte_, mWriteDataOut_, mWriteOpTagOut_, memaddrd_, opd_, ops_, pc_, rWriteAddrOut_, rWriteDataOut_, rWriteOpTagOut_, rWriteThreadOut_, sp_, status_, thread_, value_};
end
endfunction

/* --- romReadDone --- */
function [96:0] romReadDone;
input [47:0] _romReadDataIn;

// tokens
reg tInstructionReady_;

// local storage
reg [47:0] romReadDataIn_;
reg [47:0] ir_;

begin
	// init token outputs
	tInstructionReady_ = 1'b0;

	// locals init
	romReadDataIn_ = _romReadDataIn;

	// begin functional code
	ir_ = romReadDataIn_;
	tInstructionReady_ = 1'b1;

	romReadDone = {tInstructionReady_, ir_, romReadDataIn_};
end
endfunction

/* --- retiSrPopDone --- */
function [151:0] retiSrPopDone;
input [47:0] _ir;
input [15:0] _mReadAddrOut;
input _mReadByte;
input [2:0] _mReadOpTagOut;
input [15:0] _ops;
input [15:0] _pc;
input [15:0] _sp;
input [15:0] _status;
input [1:0] _thread;
input [15:0] _value;

// tokens
reg tRetiPcMemWait_;
reg tMemRead_;

// local storage
reg [15:0] status_;
reg [47:0] ir_;
reg [1:0] thread_;
reg [15:0] ops_;
reg mReadByte_;
reg [15:0] sp_;
reg [15:0] value_;
reg [15:0] pc_;
reg [15:0] mReadAddrOut_;
reg [2:0] mReadOpTagOut_;

begin
	// init token outputs
	tRetiPcMemWait_ = 1'b0;
	tMemRead_ = 1'b0;

	// locals init
	status_ = _status;
	ir_ = _ir;
	thread_ = _thread;
	ops_ = _ops;
	mReadByte_ = _mReadByte;
	sp_ = _sp;
	value_ = _value;
	pc_ = _pc;
	mReadAddrOut_ = _mReadAddrOut;
	mReadOpTagOut_ = _mReadOpTagOut;

	// begin functional code
	status_ = value_;
	mReadAddrOut_ = sp_;
	mReadByte_ = 1'd0;
	mReadOpTagOut_ = 3'd6;
	tMemRead_ = 1'b1;
	tRetiPcMemWait_ = 1'b1;
	sp_ = (sp_ + 16'd2);

	retiSrPopDone = {tMemRead_, tRetiPcMemWait_, ir_, mReadAddrOut_, mReadByte_, mReadOpTagOut_, ops_, pc_, sp_, status_, thread_, value_};
end
endfunction

/* --- rdRegDone --- */
function [226:0] rdRegDone;
input [47:0] _ir;
input [15:0] _mReadAddrOut;
input _mReadByte;
input [2:0] _mReadOpTagOut;
input [15:0] _mWriteAddrOut;
input _mWriteByte;
input [15:0] _mWriteDataOut;
input [2:0] _mWriteOpTagOut;
input [15:0] _opd;
input [15:0] _ops;
input [15:0] _pc;
input [3:0] _rWriteAddrOut;
input [15:0] _rWriteDataOut;
input [1:0] _rWriteOpTagOut;
input [1:0] _rWriteThreadOut;
input [15:0] _sp;
input [15:0] _status;
input [1:0] _thread;
input [15:0] _value;

// tokens
reg tRdRegIsDone_;

// local storage
reg [15:0] status_;
reg [2:0] mWriteOpTagOut_;
reg [47:0] ir_;
reg [1:0] thread_;
reg [15:0] mWriteDataOut_;
reg [1:0] rWriteOpTagOut_;
reg [15:0] ops_;
reg [3:0] rWriteAddrOut_;
reg [15:0] value_;
reg [2:0] mReadOpTagOut_;
reg [15:0] opd_;
reg [15:0] pc_;
reg [15:0] mReadAddrOut_;
reg mReadByte_;
reg [15:0] rWriteDataOut_;
reg [15:0] mWriteAddrOut_;
reg [1:0] rWriteThreadOut_;
reg mWriteByte_;
reg [15:0] sp_;

begin
	// init token outputs
	tRdRegIsDone_ = 1'b0;

	// locals init
	status_ = _status;
	mWriteOpTagOut_ = _mWriteOpTagOut;
	rWriteAddrOut_ = _rWriteAddrOut;
	ops_ = _ops;
	thread_ = _thread;
	mWriteDataOut_ = _mWriteDataOut;
	rWriteOpTagOut_ = _rWriteOpTagOut;
	rWriteThreadOut_ = _rWriteThreadOut;
	ir_ = _ir;
	value_ = _value;
	mReadOpTagOut_ = _mReadOpTagOut;
	opd_ = _opd;
	pc_ = _pc;
	mReadAddrOut_ = _mReadAddrOut;
	rWriteDataOut_ = _rWriteDataOut;
	mWriteAddrOut_ = _mWriteAddrOut;
	mReadByte_ = _mReadByte;
	mWriteByte_ = _mWriteByte;
	sp_ = _sp;

	// begin functional code
	opd_ = value_;
	tRdRegIsDone_ = 1'b1;

	rdRegDone = {tRdRegIsDone_, ir_, mReadAddrOut_, mReadByte_, mReadOpTagOut_, mWriteAddrOut_, mWriteByte_, mWriteDataOut_, mWriteOpTagOut_, opd_, ops_, pc_, rWriteAddrOut_, rWriteDataOut_, rWriteOpTagOut_, rWriteThreadOut_, sp_, status_, thread_, value_};
end
endfunction

/* --- commit --- */
function [164:0] commit;
input [47:0] _ir;
input [15:0] _ops;
input [15:0] _pc;
input [15:0] _sp;
input [15:0] _status;
input [1:0] _thread;

// tokens
reg tDone_;

// local storage
reg [15:0] status_;
reg [47:0] ir_;
reg [1:0] thread_;
reg [15:0] ops_;
reg [1:0] threado_;
reg [15:0] sp_;
reg [15:0] sro_;
reg [15:0] pc_;
reg [15:0] pco_;
reg [15:0] spo_;

begin
	// init token outputs
	tDone_ = 1'b0;

	// locals init
	status_ = _status;
	ir_ = _ir;
	thread_ = _thread;
	ops_ = _ops;
	sp_ = _sp;
	pc_ = _pc;

	// begin functional code
	spo_ = sp_;
	sro_ = status_;
	threado_ = thread_;
	if ((Instruction_is_single(ir_) & Instruction_single_opcode(ir_) == 3'd5)) begin
		pco_ = ops_;
	end
	else begin
		pco_ = pc_;
	end
	tDone_ = 1'b1;

	commit = {tDone_, ir_, ops_, pc_, pco_, sp_, spo_, sro_, status_, thread_, threado_};
end
endfunction

/* --- intVectorRead --- */
function [106:0] intVectorRead;
input [15:0] _pc;
input [15:0] _sp;
input [15:0] _status;
input [15:0] _value;
input [3:0] _vector;

// tokens
reg tCommit_;
reg tMemWrite_;
reg tIntPCMemWriteWait_;

// local storage
reg [15:0] status_;
reg [2:0] mWriteOpTagOut_;
reg [15:0] mWriteDataOut_;
reg mWriteByte_;
reg [15:0] sp_;
reg [15:0] value_;
reg [15:0] pc_;
reg [3:0] vector_;
reg [15:0] mWriteAddrOut_;

begin
	// init token outputs
	tCommit_ = 1'b0;
	tMemWrite_ = 1'b0;
	tIntPCMemWriteWait_ = 1'b0;

	// locals init
	status_ = _status;
	sp_ = _sp;
	value_ = _value;
	pc_ = _pc;
	vector_ = _vector;

	// begin functional code
	pc_ = value_;
	if (vector_ != 4'd15) begin
		sp_ = (sp_ - 16'd2);
		mWriteAddrOut_ = sp_;
		mWriteDataOut_ = pc_;
		mWriteByte_ = 1'd0;
		mWriteOpTagOut_ = 3'd2;
		tMemWrite_ = 1'b1;
		tIntPCMemWriteWait_ = 1'b1;
	end
	else begin
		status_ = 16'd0;
		tCommit_ = 1'b1;
	end

	intVectorRead = {tCommit_, tIntPCMemWriteWait_, tMemWrite_, mWriteAddrOut_, mWriteByte_, mWriteDataOut_, mWriteOpTagOut_, pc_, sp_, status_, value_, vector_};
end
endfunction

