
`define EMPTY		2'd0
`define HALF		2'd1
`define FULL		2'd2

module token_ff(clk, reset, valid_in, valid_out, stall_in, stall_out, data_in, data_out);	
	
	parameter DATA_WIDTH=2;	
	
	input clk;	
	input reset;	
	
	input valid_in;	
	input stall_in;	
	output valid_out;	
	output stall_out;	
	
	input [DATA_WIDTH-1:0] data_in;	
	output [DATA_WIDTH-1:0] data_out;	
	
	reg [DATA_WIDTH-1:0] data_out;	
	reg [DATA_WIDTH-1:0] data_hold;
	
	reg [1:0] state;
	
	reg valid;	
	reg stall;	
	
	assign valid_out = valid;	
	assign stall_out = stall;	
	

	reg [1:0] nextstate;
	reg Em;
	reg Es;
	
	always @ (reset or state or valid_in or stall_in) begin
		if (!reset) 
			nextstate = `EMPTY;
		else begin
			case (state)
				`EMPTY:
					begin
						if (~valid_in) begin
							nextstate = `EMPTY;		
							Em = 0;
							Es = 0;
						end
						else begin
							nextstate = `HALF;
							Em = 1;
							Es = 1;
						end
					end
				
				`HALF:
					begin
						if (~valid_in & ~stall_in) begin
							nextstate = `EMPTY;
							Em = 0;
							Es = 0;
						end
						else if (valid_in & stall_in) begin
							nextstate = `FULL;
							Em = 1;
							Es = 0;
						end
						else begin
							nextstate = `HALF;
							Em = valid_in & ~stall_in;
							Es = valid_in & ~stall_in;
						end
					end
					
				`FULL:
					begin
						if (~stall_in) begin
							nextstate = `HALF;
							Em = 0;
							Es = 1;
						end
						else begin
							nextstate = `FULL;
							Em = 0;
							Es = 0;
						end
					end
					
				default: 
					begin
						Em = 0;
						Es = 0;
					end
			endcase
		end
	end

	always @ (posedge clk) begin
		if (!reset) begin
			state <= `EMPTY;
			valid <= 1'b0;
			stall <= 1'b0;
`ifdef SIM
			data_out <= {DATA_WIDTH{1'b0}};
			data_hold <= {DATA_WIDTH{1'b0}};
`endif
		end
		else begin
			case (nextstate)
				`EMPTY: 
					begin
						valid <= 1'b0;						
						stall <= 1'b0;					
					end
						
				`HALF: 
					begin
						valid <= 1'b1;						
						stall <= 1'b0;	

						if (Em)
							data_out <= data_in;
						else if (Es)
							data_out <= data_hold;
					end
						
				`FULL: 
					begin
						valid <= 1'b1;
						stall <= 1'b1;	
						
						if (Em)
							data_hold <= data_in;
					end
			endcase

			state <= nextstate;		
		end
	end
endmodule
	
	
	
module token_ff_nd(clk, reset, valid_in, valid_out, stall_in, stall_out);	
	input clk;	
	input reset;	
	
	input valid_in;	
	input stall_in;	
	
	output valid_out;	
	output stall_out;	
	
	reg valid;	
	reg stall;	

	reg [1:0] state;
	
	assign valid_out = valid;	
	assign stall_out = stall;	

	reg [1:0] nextstate;
	
	always @ (reset or state or valid_in or stall_in) begin
		if (!reset) 
			nextstate = `EMPTY;
		else begin
			case (state)
				`EMPTY:
					begin
						if (~valid_in)
							nextstate = `EMPTY;		
						else
							nextstate = `HALF;
					end
				
				`HALF:
					begin
						if (~valid_in & ~stall_in)
							nextstate = `EMPTY;
						else if (valid_in & stall_in)
							nextstate = `FULL;
						else
							nextstate = `HALF;
					end
					
				`FULL:
					begin
						if (~stall_in)
							nextstate = `HALF;
						else
							nextstate = `FULL;
					end
			endcase
		end
	end
	
	always @ (posedge clk) begin
		if (!reset) begin
			valid <= 1'b0;
			stall <= 1'b0;
		end
		else begin
			case (nextstate)
				`EMPTY: 
					begin
						valid <= 1'b0;						
						stall <= 1'b0;					
					end
						
				`HALF: 
					begin
						valid <= 1'b1;						
						stall <= 1'b0;	
					end
						
				`FULL: 
					begin
						valid <= 1'b1;
						stall <= 1'b1;	
					end
			endcase

			state <= nextstate;		
		end
	end
endmodule