
/*********************************
    peripheral multiplier is added
    Sept.27 2009
    Wei
**********************************/


`define IDLE		2'b00
`define ACTIVE_ROM	2'b01
`define ACTIVE_RAM	2'b10
`define ACTIVE_PER  2'b11


module MemoryArb (clk,
				  reset,
				  
				  // Output to memories
			 	  addr,
				  dataIn,
				  dataOut,
				  we,
				  en,
				  b,
				  
				  addr_per,
				  dataIn_per,
				  dataOut_per,
				  we_per,
				  en_per,
				  b_per,
				  
				  
				  // Forward to ROM arbiter
				  romread_vo,
				  romread_si,
				  romread_addr,
				  romread_done_vi,
				  romread_done_so,
				  romread_data,
				  
				  // Request channel 0
				  channel0read_vi, 
				  channel0read_so,
				  channel0write_vi, 
				  channel0write_so,
				  channel0readDone_vo, 
				  channel0readDone_si,
				  channel0writeDone_vo, 
				  channel0writeDone_si,
				  channel0readAddr, 
				  channel0readData,
				  channel0readByte,
				  channel0readTagIn, 
				  channel0readTagOut,
				  channel0writeAddr, 
				  channel0writeData,
				  channel0writeByte,
				  channel0writeTagIn, 
				  channel0writeTagOut);

		  				  

	input clk;
	input reset;

	// Output to memories
	output [15:0] addr;					reg [15:0] addr;	
	input [15:0] dataIn;				
	output [15:0] dataOut;				reg [15:0] dataOut;
	output we;							reg we;
	output en;							reg en;
	output b;							reg b;
	
	// Output to peripheral
	output [15:0] addr_per;					reg [15:0] addr_per;	
	input [15:0] dataIn_per;				
	output [15:0] dataOut_per;				reg [15:0] dataOut_per;
	output we_per;							reg we_per;
	output en_per;							reg en_per;
	output b_per;							reg b_per;
	
	// Request channel 0
	input channel0read_vi; 
	output channel0read_so;				reg channel0read_so;
	input channel0write_vi; 
	output channel0write_so;			reg channel0write_so;		
	
	output channel0readDone_vo; 		reg channel0readDone_vo;
	input channel0readDone_si;
	output channel0writeDone_vo; 		reg channel0writeDone_vo;
	input channel0writeDone_si;
	
	input [15:0] channel0readAddr; 
	output [15:0] channel0readData;		reg [15:0] channel0readData;
	input channel0readByte;
	
	input [2:0] channel0readTagIn; 
	output [2:0] channel0readTagOut;	reg [2:0] channel0readTagOut;
	input [15:0] channel0writeAddr; 
	input [15:0] channel0writeData;
	input channel0writeByte;
	input [2:0] channel0writeTagIn; 
	output [2:0] channel0writeTagOut;	reg [2:0] channel0writeTagOut;
	
	// ROM forward channel
	output romread_vo;					//reg romread_vo;
	input romread_si;			
	output [15:0] romread_addr;			//reg [15:0] romread_addr;
	input romread_done_vi;				
	output romread_done_so;				//reg romread_done_so;
	input [15:0] romread_data;


	reg [1:0] readState;				reg [1:0] readState_next;
	reg [1:0] writeState;				reg [1:0] writeState_next;
	

	wire romread_vo = readState_next == `ACTIVE_ROM;
	wire romread_done_so = (readState == `ACTIVE_ROM) & romread_done_vi & channel0readDone_si;
	wire [15:0] romread_addr = channel0readAddr;
	
	reg ram_per_read;  reg ram_per_read_next;
	reg ram_per_write;  reg ram_per_write_next;
	
	
	always @ (*) begin
		if (!reset) begin
			readState_next = `IDLE;
			writeState_next = `IDLE;
			ram_per_read_next = 0;
			ram_per_write_next = 0;
		end
		else begin
			/* RAM INTERFACE */
			addr = 16'hXXXX;
			dataOut = 16'hXXXX;
			en = 1'b0;
			b = 1'b0;
			we = 1'b0;
			
			/* PERIPHERAL INTERFACE */
			addr_per = 16'hXXXX;
			dataOut_per = 16'hXXXX;
			en_per = 1'b0;
			b_per = 1'b0;
			we_per = 1'b0;
			
			
			readState_next = readState;
			writeState_next = writeState;
			
			ram_per_read_next = ram_per_read;
			ram_per_write_next = ram_per_write;
			
			


			channel0write_so = 1'b0;
			if ((channel0write_vi || writeState == `ACTIVE_RAM) && channel0writeDone_si)
				channel0write_so = 1'b1;
				
			channel0read_so = 1'b0;
			if (channel0read_vi && channel0readDone_si)
				channel0read_so = 1'b1;
	
			// Two arrive simultaneously for same memory bank
			else if (channel0read_vi && channel0write_vi) begin
				if ((channel0readAddr >= 16'h4000 && channel0writeAddr >= 16'h4000) || (channel0readAddr < 16'h4000 && channel0writeAddr < 16'h4000))
					channel0read_so = 1'b1;
			end
			// Read arrives while pending write is stalled
			else if (channel0read_vi && (writeState == `ACTIVE_RAM && channel0readAddr < 16'h4000 && (channel0write_vi | channel0writeDone_si)))
				channel0read_so = 1'b1;

			else if (readState == `ACTIVE_ROM && channel0read_vi && romread_si)
				channel0read_so = 1'b1;
				
				

			if (readState == `ACTIVE_RAM) begin
			    if(ram_per_read)
    				channel0readData = dataIn_per;	// latched by Peripheral module
       			else
       				channel0readData = dataIn;	// latched by RAM module	

				if (!channel0readDone_si)
					readState_next = `IDLE;
			    end
			else if (readState == `ACTIVE_ROM) begin
				channel0readData = romread_data; // latched by ROM module

				if (romread_done_vi && !channel0readDone_si) begin
					readState_next = `IDLE;
				end
			end
			
			
			// Only state update for processing write requests (done in synchronous or continuous assignment)
			if (writeState != `IDLE) begin
				if (!channel0writeDone_si)
					writeState_next = `IDLE;
			end
	
			// Process on next state setup to forward automatically
			if (channel0write_vi) begin
				// If write channel is idle or is finishing on this cycle
				if (writeState == `IDLE || !channel0write_so) begin
					// RAM -- no ROM (read-only)
					if (channel0writeAddr < 16'h4000) begin
						writeState_next = `ACTIVE_RAM;	// safety for erroneous software writes
						// Writes can pass active reads since data outputs are latched
						addr = channel0writeAddr;
						dataOut = channel0writeData;
						en = 1'b1;
						b = channel0writeByte;
						we = 1'b1;
						
						addr_per = channel0writeAddr;
						dataOut_per = channel0writeData;
						en_per = 1'b1;
						b_per = channel0writeByte;
						we_per = 1'b1;
						
						ram_per_write_next = 0;
						if(channel0writeAddr == 16'h0132 || channel0writeAddr == 16'h0138)
						    ram_per_write_next = 1;
						
					end
					else begin
						writeState_next = `ACTIVE_ROM;
					end
				end
			end
			
			if (channel0read_vi) begin
				if (readState == `IDLE || !channel0read_so) begin
					// RAM
					if (channel0readAddr < 16'h4000) begin
						if ((writeState != `ACTIVE_RAM || !channel0write_so) && (!channel0write_vi || channel0writeAddr >= 16'h4000)) begin
							readState_next = `ACTIVE_RAM;
							addr = channel0readAddr;
							en = 1'b1;
							b = channel0readByte;
							
							addr_per = channel0readAddr;
							en_per = 1'b1;
							b_per = channel0readByte;
							
							ram_per_read_next = 0;
    						if(channel0readAddr == 16'h013a)
						        ram_per_read_next = 1;
							
						end
					end
					else begin
						// write cannot be active on ROM
						readState_next = `ACTIVE_ROM;
					end
				end
			end
		end


	end
	

	
	always @ (posedge clk) begin
		if (!reset) begin
			readState <= `IDLE;
			writeState <= `IDLE;

			channel0readDone_vo <= 1'b0;
			channel0writeDone_vo <= 1'b0;
		end
		else begin
			if (readState_next != `IDLE)
				channel0readDone_vo <= 1'b1;
			else
				channel0readDone_vo <= 1'b0;

			if (writeState_next != `IDLE)
				channel0writeDone_vo <= 1'b1;
			else
				channel0writeDone_vo <= 1'b0;
	
				
			if (readState_next != `IDLE && !channel0read_so)
				channel0readTagOut <= channel0readTagIn;
			if (writeState_next != `IDLE && !channel0write_so) begin
				channel0writeTagOut <= channel0writeTagIn;
			end
				
			readState <= readState_next;
			writeState <= writeState_next;
			
			ram_per_read <= ram_per_read_next;
			ram_per_write <= ram_per_write_next;
		end
	end
	
endmodule


module MULTIP (clk, en, addr, dataIn, dataOut, we,b);

	parameter ADDRESS_WIDTH = 14;
	parameter DATA_WIDTH = 16;		

	input clk;
	input en;
	input [ADDRESS_WIDTH-1:0] addr;
	input [DATA_WIDTH-1:0] dataIn;
	output [DATA_WIDTH-1:0] dataOut;
	input we;
	input b;
	
	reg [DATA_WIDTH-1:0] dataOut;
	
    reg [DATA_WIDTH-1:0] dataInA;
	reg [DATA_WIDTH-1:0] dataInB;

	always @ (posedge clk) begin
 
        if (en) begin
            if (we) begin
                if(addr == 16'h0132)
                    dataInA = dataIn;
                if(addr == 16'h0138)
                    dataInB = dataIn;
            end
            else begin
                if(addr == 16'h013a)
                    dataOut = dataInA * dataInB;
            end
        end

	end
endmodule
	
	
	
	
	
module RAM (clk,
            rst,
			en,
			addr,
			dataIn,
			dataOut,
			we,
			b);
	
	parameter ADDRESS_WIDTH = 14;
	parameter DATA_WIDTH = 16;
	parameter DEPTH = 7296;
	
	input clk;
	input rst;
	
	input en;
	input b;
	
	input [DATA_WIDTH-1:0] dataIn;
	output [DATA_WIDTH-1:0] dataOut;
	
	input [ADDRESS_WIDTH-1:0] addr;
	
	input we;

	reg cfg;
	
	wire enh = en & (addr[0] | ~b);
	wire [(DATA_WIDTH/2)-1:0] datainh = b ? dataIn[(DATA_WIDTH/2)-1:0] : dataIn[DATA_WIDTH-1:(DATA_WIDTH/2)];
	wire [(DATA_WIDTH/2)-1:0] dataouth;
	RAM8 #(ADDRESS_WIDTH-1, DATA_WIDTH/2, DEPTH) hi (
			 .clk(clk),
			 .rst(rst),
			 .en(enh),
			 .addr(addr[ADDRESS_WIDTH-1:1]),
			 .dataIn(datainh),
			 .dataOut(dataouth),
			 .we(we));

	wire enl = en & ~addr[0];
	wire [(DATA_WIDTH/2)-1:0] datainl = dataIn[(DATA_WIDTH/2)-1:0];
	wire [(DATA_WIDTH/2)-1:0] dataoutl;
	RAM8 #(ADDRESS_WIDTH-1, DATA_WIDTH/2, DEPTH) lo (
			 .clk(clk),
			 .rst(rst),
			 .en(enl),
			 .addr(addr[ADDRESS_WIDTH-1:1]),
			 .dataIn(datainl),
			 .dataOut(dataoutl),
			 .we(we));

	wire [DATA_WIDTH-1:0] dataOut = cfg ? {dataouth, dataoutl} : {8'd0, dataouth};
	
			 
	always @ (posedge clk) begin
		// Read
		if (en && !we) begin
			cfg <= ~b | ~addr[0];
		end
	end
endmodule
	


module RAM8 (clk, rst, en, addr, dataIn, dataOut, we);

	parameter ADDRESS_WIDTH = 8;
	parameter DATA_WIDTH = 8;
	parameter DEPTH = 256;			

	input clk;
    input rst;
	input en;
	input [ADDRESS_WIDTH-1:0] addr;
	input [DATA_WIDTH-1:0] dataIn;
	output [DATA_WIDTH-1:0] dataOut;
	input we;
	
	
	    integer i;


	reg [DATA_WIDTH-1:0] data [DEPTH-1:0];
	
	reg [DATA_WIDTH-1:0] dataOut;

	always @ (posedge clk) begin

        if(!rst) begin
            for(i=0; i< DEPTH; i = i+1)
                data[i] = 0;
        end	
        else begin
	    	if (en) begin
		    	if (we)
		    		data[addr] = dataIn;
		    	else
			    	dataOut = data[addr];
		    end
        end
	end
endmodule
