/*********************************
    peripheral multiplier is added
    Sept.27 2009
    Wei
**********************************/

`define SIM

module proc(clk, reset, 
			interrupt, ivec, iservice);


	input clk;
	input reset;
	input interrupt;
	input ivec;
	output iservice;
	
	//////////////////////////////////////////////////////////////////////////////////////////////
	// Context
	wire [15:0] pc;
	wire [15:0] sp;
	wire [15:0] sr;
	wire [1:0] thread;
	
	//////////////////////////////////////////////////////////////////////////////////////////////
	// Processor Interfaces
	// Pipeline
	wire cycle_vi;
	wire cycle_so;
	
	wire done_vo;
	//reg done_si;

	wire [15:0] pc_;
	wire [15:0] sp_;
	wire [15:0] sr_;
	wire [1:0] thread_;

	// Interrupt
	wire interrupt_vi;
	wire interrupt_so;
	
	wire [3:0] vector;
	reg iservice;
	//
	//////////////////////////////////////////////////////////////////////////////////////////////
	
	//////////////////////////////////////////////////////////////////////////////////////////////
	// Register Interface
	wire regread_vo;
	wire regread_si;
	
	wire [3:0] regRAddrOut;
	wire [1:0] regRThreadOut;
	wire [15:0] regRDataIn;
	wire regRTagIn;
	wire regRTagOut;

	wire regwrite_vo;
	wire regwrite_si;
	
	wire [3:0] regWAddrOut;
	wire [1:0] regWThreadOut;
	wire [15:0] regWDataOut;
	wire [1:0] regWTagIn;
	wire [1:0] regWTagOut;	
	
	wire regreaddone_vi;
	wire regreaddone_so;
	wire regwritedone_vi;
	wire regwritedone_so;
	//
	//////////////////////////////////////////////////////////////////////////////////////////////
	
	//////////////////////////////////////////////////////////////////////////////////////////////
	// Memory Interface
	// channel 0
	wire instread_vo;
	wire instread_si;
	
	wire [15:0] instRAddr;
	wire [31:0] instRData;
	wire [1:0]	instRDataSize;
	
	wire instreaddone_vi;
	wire instreaddone_so;

	// channel 0
	wire memread_vo;
	wire memread_si;
	wire memwrite_vo;
	wire memwrite_si;
	
	wire [15:0] memRAddrOut;
	wire [15:0] memRDataIn;
	wire memRByte;
	
	wire [2:0] memRTagOut;
	wire [2:0] memRTagIn;
	
	wire [15:0] memWAddrOut;
	wire [15:0] memWDataOut;
	wire memWByte;
	
	wire [2:0] memWTagOut;
	wire [2:0] memWTagIn;
	
	wire memreaddone_vi;
	wire memreaddone_so;
	wire memwritedone_vi;
	wire memwritedone_so;
	//
	//////////////////////////////////////////////////////////////////////////////////////////////
	
	//////////////////////////////////////////////////////////////////////////////////////////////
	// ROM Interface
	wire romread_vo;
	wire romread_si;
	
	wire [15:0] romRAddrOut;
	wire [47:0] romRDataIn;
	
	wire romreaddone_vi;
	wire romreaddone_so;
	//
	//////////////////////////////////////////////////////////////////////////////////////////////


	ExecUnit unit1(.i_clk(clk),
				   .i_reset(reset),
				   
				   .i_tCycle_valid_in(cycle_vi),
				   .o_tCycle_stall_out(cycle_so),

				   .i_pc(pc),
				   .i_sp(sp),
				   .i_sr(sr),
				   .i_thread(thread),

				   .i_tInterrupt_valid_in(interrupt_vi),
				   .o_tInterrupt_stall_out(interrupt_so),
				   
				   .i_vector(vector),

				   .o_tDone_valid_out(done_vo),
				   .i_tDone_stall_in(1'b0),
				   
				   .o_pco(pc_),
				   .o_spo(sp_),
				   .o_sro(sr_),
				   .o_threado(thread_),
				   
				   // register interface
				   .o_tRegRead_valid_out(regread_vo),
				   .i_tRegRead_stall_in(regread_si),
				   
				   .o_rReadAddrOut(regRAddrOut),
				   .o_rReadThreadOut(regRThreadOut),
				   .i_rReadDataIn(regRDataIn),
				   .o_rReadOpTagOut(regRTagOut),
				   .i_rReadOpTagIn(regRTagIn),

				   .o_tRegWrite_valid_out(regwrite_vo),
				   .i_tRegWrite_stall_in(regwrite_si),

				   .o_rWriteAddrOut(regWAddrOut),
				   .o_rWriteDataOut(regWDataOut),
				   .o_rWriteThreadOut(regWThreadOut),
				   .o_rWriteOpTagOut(regWTagOut),
				   .i_rWriteOpTagIn(regWTagIn),
				   
				   // -responses
				   .i_tRegReadDone_valid_in(regreaddone_vi),
				   .o_tRegReadDone_stall_out(regreaddone_so),
				   .i_tRegWriteDone_valid_in(regwritedone_vi),
				   .o_tRegWriteDone_stall_out(regwritedone_so),
				   
				   
				   // memory interface
				   .o_tMemRead_valid_out(memread_vo),
				   .i_tMemRead_stall_in(memread_si),
				   
				   .o_mReadAddrOut(memRAddrOut),
				   .o_mReadByte(memRByte),
				   .i_mReadDataIn(memRDataIn),

				   .o_mReadOpTagOut(memRTagOut),
				   .i_mReadOpTagIn(memRTagIn),

				   .o_tMemWrite_valid_out(memwrite_vo),
				   .i_tMemWrite_stall_in(memwrite_si),
				   
				   .o_mWriteAddrOut(memWAddrOut),
				   .o_mWriteDataOut(memWDataOut),
				   .o_mWriteByte(memWByte),
				   
				   .o_mWriteOpTagOut(memWTagOut),
				   .i_mWriteOpTagIn(memWTagIn),
				
				   // -responses
				   .i_tMemReadDone_valid_in(memreaddone_vi),
				   .o_tMemReadDone_stall_out(memreaddone_so),
				   .i_tMemWriteDone_valid_in(memwritedone_vi),
				   .o_tMemWriteDone_stall_out(memwritedone_so),
				   
				   
				   // rom interface
				   .o_tRomRead_valid_out(romread_vo),
				   .i_tRomRead_stall_in(romread_si),
				   
				   .o_romReadAddrOut(romRAddrOut),
				   .i_romReadDataIn(romRDataIn),
				   
				   // -response
				   .i_tRomReadDone_valid_in(romreaddone_vi),
				   .o_tRomReadDone_stall_out(romreaddone_so));
				   

	RegisterLI #(.ADDRESS_WIDTH(6), .DATA_WIDTH(16), .DEPTH(64))
			 registerFile (.clk(clk),
						  .reset(reset), 

						  // processor port
						  .channel0read_vi(regread_vo), 
						  .channel0read_so(regread_si),
						  .channel0write_vi(regwrite_vo), 
						  .channel0write_so(regwrite_si),
						  .channel0readDone_vo(regreaddone_vi), 
						  .channel0readDone_si(regreaddone_so),
						  .channel0writeDone_vo(regwritedone_vi), 
						  .channel0writeDone_si(regwritedone_so),
						  .channel0readAddr({regRThreadOut, regRAddrOut}), 
						  .channel0readData(regRDataIn),
						  .channel0readTagIn(regRTagOut), 
						  .channel0readTagOut(regRTagIn),
						  .channel0writeAddr({regWThreadOut, regWAddrOut}), 
						  .channel0writeData(regWDataOut),
						  .channel0writeTagIn(regWTagOut), 
						  .channel0writeTagOut(regWTagIn));

	
	///////////////////////////////////////////////////////////////////////////////////////////////
	/// MEMORY ARBITRATION

	wire [12:0] romAddrLo;
	wire [12:0] romAddrHi;
	wire [31:0] romDataLo;
	wire [31:0] romDataHi;
	wire romFRead_vo;
	wire romFRead_si;
	wire [15:0] romFReadAddr;
	wire romFReadDone_vi;
	wire romFReadDone_so;
	wire [47:0] romFReadData;
	ROMArbiter romArb  (.clk(clk),
						.reset(reset),
						
						// connected to passthrough from normal memory space reads
						.ch0vi(romFRead_vo),
						.ch0so(romFRead_si),
						.ch0addr(romFReadAddr),
						.ch0data(romFReadData),
						.ch0vo(romFReadDone_vi),
						.ch0si(romFReadDone_so),
						
						// give fetch a lower priority so that reads don't back up the pipe
						.ch1vi(romread_vo),
						.ch1so(romread_si),
						.ch1addr(romRAddrOut),
						.ch1data(romRDataIn),
						.ch1tagin(1'b0),
						.ch1tagout(),
						
						.ch1vo(romreaddone_vi),
						.ch1si(romreaddone_so),
						   
						.addrlo(romAddrLo),
						.addrhi(romAddrHi),
						.datalo(romDataLo),
						.datahi(romDataHi));

	ROM #(13, 32, 8192, "rom_lo.data")
	    rom_lo (.en(1),
	    	    .addr(romAddrLo),
	    	    .dataOut(romDataLo));
	ROM #(13, 32, 8192, "rom_hi.data")
	    rom_hi (.en(1),
	    	    .addr(romAddrHi),
	    	    .dataOut(romDataHi));
	///					  
	///////////////////////////////////////////////////////////////////////////////////////////////

					
					
	///////////////////////////////////////////////////////////////////////////////////////////////
	/// MEMORY ARBITRATION
	wire ram_en;
	wire ram_we;
	wire ram_b;
	wire [15:0] ram_addr;
	wire [15:0] ram_data_in;
	wire [15:0] ram_data_out;
	wire [15:0] per_data_out;
	MemoryArb arb (.clk(clk),
				   .reset(reset),
				   
				   .addr(ram_addr),
				   .dataIn(ram_data_out),
				   .dataOut(ram_data_in),
				   .we(ram_we),
				   .en(ram_en),
				   .b(ram_b),
				   
				   .addr_per(ram_addr),
				   .dataIn_per(per_data_out),
				   .dataOut_per(ram_data_in),
				   .we_per(ram_we),
				   .en_per(ram_en),
				   .b_per(ram_b),

				   .romread_vo(romFRead_vo),
				   .romread_si(romFRead_si),
				   .romread_addr(romFReadAddr),
				   .romread_done_vi(romFReadDone_vi),
				   .romread_done_so(romFReadDone_so),
				   .romread_data(romFReadData[15:0]),
				   
				   .channel0read_vi(memread_vo), 
				   .channel0read_so(memread_si),
				   .channel0write_vi(memwrite_vo), 
				   .channel0write_so(memwrite_si),
				   .channel0readDone_vo(memreaddone_vi), 
				   .channel0readDone_si(memreaddone_so),
				   .channel0writeDone_vo(memwritedone_vi), 
				   .channel0writeDone_si(memwritedone_so),
				   .channel0readAddr(memRAddrOut), 
				   .channel0readByte(memRByte),
				   .channel0readData(memRDataIn),
				   .channel0readTagIn(memRTagOut), 
				   .channel0readTagOut(memRTagIn),
				   .channel0writeAddr(memWAddrOut), 
				   .channel0writeData(memWDataOut),
				   .channel0writeByte(memWByte),
				   .channel0writeTagIn(memWTagOut), 
				   .channel0writeTagOut(memWTagIn));			   		   
	///					  
	///////////////////////////////////////////////////////////////////////////////////////////////
	/// MEMORY SWITCHING
	wire ram_enable = ram_en && ram_addr >= 16'h1100 && ram_addr <= 16'h38FF;
	
	/// MEMORY ELEMENTS (MPS430-F1611)
	/// ROM 0x8000 - 0xFFFF

	/// ROM 0x4000 - 0x7FFF
	    	 
	/// RAM 0x1900 - 0x38FF (Extended)
	/// RAM 0x1100 - 0x18FF (Mirrored) -- Ignore (compatibility only)
	RAM #(14, 16, 7296)	//// IMPLEMENT FROM 0x0000 - 0x38FF
		ram (.clk(clk),
		     .rst(reset),
			 .en(ram_enable),
			 .addr(ram_addr[13:0]),
			 .dataIn(ram_data_in),
			 .dataOut(ram_data_out),
   			 .b(ram_b),
			 .we(ram_we));

	/// RAM 0x0200 - 0x09FF (Mirrored) -- Ignore (compatibility only)
					
	/// SFR 0x0000 - 0x00FF (8-bit)
	/// SFR 0x0100 - 0x01FF (16-bit)
	/// PERIPHERAL MULTIPLIER
	wire multip_enable = ram_en && ram_addr >= 16'h0100 && ram_addr <= 16'h10FF;
	MULTIP #(14, 16)	//// IMPLEMENT FROM 0x0000 - 0x01FF
		multip (.clk(clk),
			 .en(multip_enable),
			 .addr(ram_addr[13:0]),
			 .dataIn(ram_data_in),
			 .dataOut(per_data_out),
   			 .b(ram_b),
			 .we(ram_we));

	Logger logger(.clk(clk),
				  .en(ram_en),
				  .we(ram_we),
				  .address(ram_addr),
				  .data(ram_data_in));
	///
	///////////////////////////////////////////////////////////////////////////////////////////////


	///////////////////////////////////////////////////////////////////////////////////////////////
	///
	Threader4 threadu(.clk(clk),
				  	.reset(reset),
				  	
				    .en(ram_en),
				    .we(ram_we),
				    .address(ram_addr),
				    .datain(ram_data_in),
				    
				    .thread(thread),
					.pc(pc),
					.sp(sp),
					.sr(sr),

				    .run_vo(cycle_vi),
				    .run_si(cycle_so),
				    
				    .int_vo(interrupt_vi),
				    .int_si(interrupt_so),
				    .vector(vector),
				    
				    .done(done_vo),
				    .pcn(pc_),
				    .spn(sp_),
				    .srn(sr_),
				    .threadn(thread_));
				  
	///
	///////////////////////////////////////////////////////////////////////////////////////////////

endmodule
