
module RegisterLI (clk, reset, 
			   
			       /* CHANNEL 0 */
			       channel0read_vi, channel0read_so,
			       channel0write_vi, channel0write_so,

			       channel0readDone_vo, channel0readDone_si,
			       channel0writeDone_vo, channel0writeDone_si,
			   
			       channel0readAddr, channel0readData,
			       channel0readTagIn, channel0readTagOut,
			   
			       channel0writeAddr, channel0writeData,
			       channel0writeTagIn, channel0writeTagOut
			     
			       );

			   
	/* PARAMETERS */			   
	parameter ADDRESS_WIDTH = 4;		   
	parameter DATA_WIDTH = 16;			   
	parameter DEPTH = 16;
	
	
	input clk;
	input reset;
			

    /* CHANNEL 0 */
	input channel0read_vi;
	output channel0read_so;						//reg channel0read_so;
	input channel0write_vi;
	output channel0write_so;					//reg channel0write_so;
	
	output channel0readDone_vo;					reg channel0readDone_vo;
	input channel0readDone_si;
	output channel0writeDone_vo;				reg channel0writeDone_vo;
	input channel0writeDone_si;
	
	input [ADDRESS_WIDTH-1:0] channel0readAddr;
	output [15:0] channel0readData;				reg [15:0] channel0readData;

	input channel0readTagIn;
	output channel0readTagOut;					reg channel0readTagOut;

	input [ADDRESS_WIDTH-1:0] channel0writeAddr;
	input [15:0] channel0writeData;
	
	input [1:0] channel0writeTagIn;
	output [1:0] channel0writeTagOut;			reg [1:0] channel0writeTagOut;
	

	reg [DATA_WIDTH-1:0] data [DEPTH-1:0];
	
	assign channel0read_so = channel0read_vi & channel0readDone_vo & channel0readDone_si;
	assign channel0write_so = channel0write_vi & channel0writeDone_vo & channel0writeDone_si;


`ifdef SIM	
	integer i;

	reg ch0rvifu;
	reg ch0wvifu;

	always @ (clk or channel0read_vi or channel0write_vi) begin
		ch0rvifu = channel0read_vi;
		ch0wvifu = channel0write_vi;
	end	
`endif

	always @ (posedge clk) begin
		if (!reset) begin
`ifdef SIM
			for (i = 0; i < DEPTH; i = i + 1)
				data[i] <= {DATA_WIDTH{1'd0}};
`endif

			channel0readDone_vo <= 0;
			channel0writeDone_vo <= 0;
		end
		else begin
`ifdef SIM
			if (ch0wvifu) begin
`else
			if (channel0write_vi) begin
`endif
				if (!channel0writeDone_si || !channel0writeDone_vo) begin
					// Perform write
					data[channel0writeAddr] <= channel0writeData;				
					channel0writeTagOut <= channel0writeTagIn;

					// Signal completion
					channel0writeDone_vo <= 1;
				end
			end
			else if (!channel0writeDone_si) begin
				channel0writeDone_vo <= 0;
			end

`ifdef SIM
			if (!channel0readDone_si)
				channel0readData <= {DATA_WIDTH{1'bx}};
`endif


`ifdef SIM
			if (ch0rvifu) begin
`else
			if (channel0read_vi) begin
`endif
				if (!channel0readDone_si || !channel0readDone_vo) begin
					channel0readData <= data[channel0readAddr];
					channel0readTagOut <= channel0readTagIn;
					
					channel0readDone_vo <= 1;
				end	
			end
			else if (!channel0readDone_si) begin
				channel0readDone_vo <= 0;
			end
		end
	end

endmodule
	
	
	
	
	
	
				   
